@@ -1,11 +1,11 @@
-#undef PACKAGE
-#undef VERSION
 #undef HAVE_LIBSM
 #undef HAVE_CATGETS
 #undef HAVE_GETTEXT
+#undef ENABLE_NLS
+
 #undef HAVE_LC_MESSAGES
 #undef HAVE_STPCPY
-#undef ENABLE_NLS
+
 #undef HAVE_GETHOSTBYNAME_R_GLIBC
 #undef HAVE_GETHOSTBYNAME_R_SOLARIS
 #undef HAVE_GETHOSTBYNAME_R_HPUX
@@ -18,4 +18,6 @@
 #undef WITH_IPV6
 #undef HAVE_XMMS
 #undef MACOSX
-#undef USING_STD_SSTREAM
+#undef SOLARIS
+#undef HAVE_STD_SSTREAM
+#undef HAVE_SOCKLEN_T
