@@ -25,6 +25,8 @@
  *
  * Date             Modified by     Description of modification
  * 01/20/2002       IBM Corp.       Updated to libjudo 1.1.1
+ * 2002-03-05       IBM Corp.       Updated to libjudo 1.1.5
+ * 2002-03-09       IBM Corp.       Catch libjudo's ParserError
  */
 
 
@@ -92,17 +94,19 @@ void Session::connect(const string& server, AuthType atype,
 
 bool Session::disconnect()
 {
+     bool success = false;
+
      // only disconnect if we have received the starting stream tag to prevent the tagstream from throwing
      // an exception
      if ((_ConnState != csNotConnected) && _StreamStart)
      {
-	  *this >> "</stream:stream>";
-          return true;
+       *this << "</stream:stream>";
+       success = true;
      }
      _ConnState = csNotConnected;
      _StreamStart = false;
 
-     return false;
+     return success;
 }
 
 // ---------------------------------------------------------
@@ -160,7 +164,18 @@ string Session::getDigest()
 void Session::push(const char* data, int datasz)
 {
      evtRecvXML(data);
-     ElementStream::push(data, datasz);
+     try {
+	  ElementStream::push(data, datasz);
+     } catch (const ElementStream::exception::ParserError& error) {
+	  // We had a parser error, so nothing's going to be happy
+	  evtXMLParserError(error.getCode(), error.getMessage());
+
+	  // Prevent loops :)
+	  _ConnState = csNotConnected;
+	  _StreamStart = false;
+
+	  disconnect();
+     }
 }
 
 void Session::registerIQ(const string& id, ElementCallbackFunc f)
@@ -217,23 +232,23 @@ void Session::OnAuthTypeReceived(const Element& t)
 {
      // Grab the supported auth types and login using appropriate one
      Session::AuthType atype = Session::atPlaintextAuth;
-     Element* query = NULL;
+     const Element* query = NULL;
      if (!t.empty())
      {
 	  query = t.findElement("query");
 	  if (query != NULL)
 	  {
 	       // Plaintext auth
-	       Element* password = query->findElement("password");
+	       const Element* password = query->findElement("password");
 	       if (password != NULL)
 		    atype = Session::atPlaintextAuth;
 	       // Digest auth
-	       Element* digest = query->findElement("digest");
+	       const Element* digest = query->findElement("digest");
 	       if (digest != NULL)
 		    atype = Session::atDigestAuth;
 	       // Zero-Knowledge
-	       Element* token = query->findElement("token");
-	       Element* sequence = query->findElement("sequence");
+	       const Element* token = query->findElement("token");
+	       const Element* sequence = query->findElement("sequence");
 	       if (token != NULL && sequence != NULL)
 		    atype = Session::at0kAuth;  // Add 0k support
 	  }
@@ -281,8 +296,8 @@ void Session::sendLogin(Session::AuthType atype, const Element* squery)
 	       break;
 	  case at0kAuth:
 	       string hashA = shahash(_Password.c_str());
-	       string token = squery->findElement_getCDATA("token");
-	       int seq = atoi(squery->findElement_getCDATA("sequence").c_str());
+	       string token = squery->getChildCData("token");
+	       int seq = atoi(squery->getChildCData("sequence").c_str());
 	       string hash = shahash(string(hashA + token).c_str());
 	       for (int i = 0; i < seq; i++)
 		    hash = shahash(hash.c_str());
@@ -451,7 +466,7 @@ void Session::handleIQ(Element& t)
 	  {
 	       string name, ver, os;
 	       evtOnVersion(name, ver, os);
-	       ver += " (powered by jabberoo 1.1.0)";
+	       ver += " (powered by jabberoo 1.1.2)";
 	       Packet iq("iq");
 	       iq.getBaseElement().putAttrib("type", "result");
 	       iq.setTo(t.getAttrib("from"));
@@ -499,9 +514,15 @@ void Session::handleIQ(Element& t)
 	       // locale-dependent
 	       //strftime(timestr, 1024, "%d %b %Y %H:%M:%S", loctime);
 	       strftime(timestr, 1024, "%c", loctime);
-	       query->addElement("display", timestr);
+	       string local= string(timestr); // Local time string
 	       strftime(timestr, 1024, "%Z", loctime);
-	       query->addElement("tz", timestr);
+	       string tz = string(timestr); // Time zone
+               if (local.find(tz)!=string::npos) // Don't duplicate timezone
+                       local.erase(local.find(tz));
+	       evtOnTime(local, tz);
+
+	       query->addElement("display", local);
+	       query->addElement("tz", tz);
 	       *this << iq;
 #endif
 	  }
@@ -531,7 +552,7 @@ void Session::IQHandler_Auth(const Element& t)
      else
      {
 	  // Handle errors
-	  Element* error = t.findElement("error");
+	  const Element* error = t.findElement("error");
 	  if (error != NULL)
 	  {
 	       _ConnState = Session::csAuthReq;
@@ -551,7 +572,7 @@ void Session::IQHandler_CreateUser(const Element& t)
 	  authenticate();
      else
      {
-	  Element* error = t.findElement("error");
+	  const Element* error = t.findElement("error");
 	  if (error != NULL)
 	  {
 	       evtAuthError(atoi(error->getAttrib("code").c_str()), error->getCDATA().c_str());
