@@ -82,7 +82,7 @@ AddContactDruid::AddContactDruid(Page first_page, const string& jid)
 AddContactDruid::AddContactDruid(Page first_page, Agent& agent)
      : BaseGabberWindow("AUDruid_win"),
        _protocol(0),
-       _agent((Agent*) &agent)
+       _agent(manage(new jabberoo::Agent(agent)))
 {
      init();
      _druid->set_page(*translate_page(first_page));
@@ -353,11 +353,14 @@ void AddContactDruid::on_Protocol_prepare()
      G_App->getSession().queryNamespace("jabber:iq:agents", slot(this, &AddContactDruid::handle_Protocol_iq), G_App->getCfg().get_server());
 }
 
-void AddContactDruid::on_protocol_selected(bool selected, bool registerable, bool searchable, bool gccapable, bool agents, bool transport)
+void AddContactDruid::on_protocol_selected(jabberoo::Agent* agent)
 {
-     // Set the sensitivity of the appropriate buttons
-     _druid->set_buttons_sensitive(true, transport, true);
-     _protocol = _browserProtocol->get_current_agent();
+    if (agent)
+    {
+        // Set the sensitivity of the appropriate buttons
+        _druid->set_buttons_sensitive(true, !agent->transport().empty(), true);
+        _protocol = agent;
+    }
 }
 
 void AddContactDruid::handle_Protocol_iq(const Element& iq)
@@ -372,7 +375,7 @@ void AddContactDruid::handle_Protocol_iq(const Element& iq)
      if (iq.cmpAttrib("type", "result"))
      {
 	  // Extract the query 
-	  Element* query = iq.findElement("query");
+	  const Element* query = iq.findElement("query");
 	  if (query != NULL)
 	  {
 	       // Update the agents
@@ -501,11 +504,14 @@ void AddContactDruid::on_Agents_prepare()
      G_App->getSession().queryNamespace("jabber:iq:agents", slot(this, &AddContactDruid::handle_Agents_iq), G_App->getCfg().get_server());
 }
 
-void AddContactDruid::on_agent_selected(bool selected, bool registerable, bool searchable, bool gccapable, bool agents, bool transport)
+void AddContactDruid::on_agent_selected(jabberoo::Agent* agent)
 {
-     // Set the sensitivity of the appropriate buttons
-     _druid->set_buttons_sensitive(true, searchable, true);
-     _agent = _browserSearch->get_current_agent();
+    if (agent)
+    {
+        // Set the sensitivity of the appropriate buttons
+        _druid->set_buttons_sensitive(true, agent->isSearchable(), true);
+        _agent = agent;
+    }
 }
 
 void AddContactDruid::handle_Agents_iq(const Element& iq)
@@ -520,7 +526,7 @@ void AddContactDruid::handle_Agents_iq(const Element& iq)
      if (iq.cmpAttrib("type", "result"))
      {
 	  // Extract the query 
-	  Element* query = iq.findElement("query");
+	  const Element* query = iq.findElement("query");
 	  if (query != NULL)
 	  {
 	       // Update the agents
@@ -562,6 +568,8 @@ bool AddContactDruid::on_Agents_back()
 // -------------------------------------------------------------------------------------
 void AddContactDruid::on_Search_prepare()
 {
+     if (!_agent) return;
+
      // Set the agent labels
      Gtk::Label* l = getLabel("AUDruid_search_agent_lbl");
      l->set_text(fromUTF8(l, _agent->JID()));
@@ -591,7 +599,7 @@ void AddContactDruid::handle_search_request_reply(const Element& iq)
      if (iq.cmpAttrib("type", "result"))
      {
           // Extract the query 
-          Element* query = iq.findElement("query");
+          const Element* query = iq.findElement("query");
 	  Element::const_iterator it = query->begin();
 	  
 	  _field_names.push_back("JID");
@@ -760,7 +768,7 @@ void AddContactDruid::on_search_reply(const Element& iq)
 		         {
                               string* field = &(*sit);
 			      
-			      rowdata[i] = g_strdup(fromUTF8(_results_clist, item.findElement_getCDATA(*field)).c_str());
+			      rowdata[i] = g_strdup(fromUTF8(_results_clist, item.getChildCData(*field)).c_str());
 		         }
 		         _results_clist->append_row(rowdata);
 			 for (i = 0; i < num_fields; i++)
@@ -772,11 +780,11 @@ void AddContactDruid::on_search_reply(const Element& iq)
      }
      else if (iq.cmpAttrib("type", "error"))
      {
-	  Element* error = iq.findElement("error");
+	  const Element* error = iq.findElement("error");
 	  string errormessage = string(_("Error ")) 
 	       + string(error->getAttrib("code")) 
 	       + string(":\n") 
-	       + string(iq.findElement_getCDATA("error"));
+	       + string(iq.getChildCData("error"));
 	  Gnome::Dialog* d = manage(Gnome::Dialogs::warning(errormessage.c_str()));
 	  d->set_modal(true);
 		// d is a child window of this window:
@@ -863,27 +871,27 @@ void AddContactDruid::get_vCard(const Element& t)
      if (!t.empty())
      {
 	  string curtext;
-	  Element* vCard = t.findElement("vCard");
+	  const Element* vCard = t.findElement("vCard");
 	  if (vCard != NULL)
 	  {
-	       curtext = vCard->findElement_getCDATA("NICKNAME");
+	       curtext = vCard->getChildCData("NICKNAME");
 	       if (_entNickname->get_text().empty() ||
 		   toUTF8(_entNickname, _entNickname->get_text()) == JID::getUser(_JID))
 		    _entNickname->set_text(fromUTF8(_entNickname, curtext));
-	       curtext = vCard->findElement_getCDATA("FN");
+	       curtext = vCard->getChildCData("FN");
 	       Gtk::Label* l = getLabel("AUDruid_nickname_FullName_lbl");
 	       l->set_text(fromUTF8(l, curtext));
-	       curtext = vCard->findElement_getCDATA("EMAIL");
+	       curtext = vCard->getChildCData("EMAIL");
 	       l = getLabel("AUDruid_nickname_eMail_lbl");
 	       l->set_text(fromUTF8(l, curtext));
-	       curtext = vCard->findElement_getCDATA("URL");
+	       curtext = vCard->getChildCData("URL");
 	       Gnome::HRef* hrefWeb = getWidget<Gnome::HRef>("AUDruid_nickname_WebSite_href");
 	       hrefWeb->set_url(fromUTF8(hrefWeb, curtext));
 	       hrefWeb->set_label(fromUTF8(hrefWeb, curtext));
-	       Element* ADR = vCard->findElement("ADR");
+	       const Element* ADR = vCard->findElement("ADR");
 	       if (ADR != NULL)
 	       {
-		    curtext = ADR->findElement_getCDATA("COUNTRY");
+		    curtext = ADR->getChildCData("COUNTRY");
 		    l = getLabel("AUDruid_nickname_Country_lbl");
 		    l->set_text(fromUTF8(l, curtext));
 	       }
