@@ -100,6 +100,10 @@ void AgentBrowser::clear()
      _ctreeAgent->freeze();
      // Clear the tree
      _ctreeAgent->clear();
+     _current_agent = NULL;
+
+     agent_selected(NULL);
+     cerr << "clear() _agent now NULL" << endl;
      _ctreeAgent->thaw();
 }
 
@@ -127,10 +131,7 @@ void AgentBrowser::on_browse_clicked()
      G_App->getCfg().saveEntryHistory(_gentServer);
      G_App->getCfg().loadEntryHistory(_gentServer);
 
-     _ctreeAgent->freeze();
-     // Clear the tree
-     _ctreeAgent->clear();
-     _ctreeAgent->thaw();
+     clear();
 
      // Send request to the server for the update
      G_App->getSession().queryNamespace("jabber:iq:agents", slot(this, &AgentBrowser::handle_agents_IQ), toUTF8(_entServer, _entServer->get_text()));
@@ -154,24 +155,16 @@ void AgentBrowser::on_tree_select(Gtk::CTree::Row r, int col)
 
      if (_current_agent != NULL)
      {
-	  agent_selected(true,
-			 _current_agent->isRegisterable(),
-			 _current_agent->isSearchable(),
-			 _current_agent->isGCCapable(),
-			 _current_agent->hasAgents(),
-			 !_current_agent->transport().empty());
+	  agent_selected(_current_agent);
+	  cerr << "tree_select() _agent now " << _current_agent->JID() << endl;
      }
 }
 
 void AgentBrowser::on_tree_unselect(Gtk::CTree::Row r, int col)
 {
      _current_agent = NULL;
-     agent_selected(false,
-		    false,
-		    false,
-		    false,
-		    false,
-		    false);
+     agent_selected(NULL);
+     cerr << "tree_unselect() _agent now NULL" << endl;
 }
 
 void AgentBrowser::on_agent_fetch_complete(bool successful, GtkCTreeNode* agentnode, GtkCTreeNode* dummynode)
@@ -214,7 +207,7 @@ void AgentBrowser::handle_agents_IQ(const Element& iq)
      if (iq.cmpAttrib("type", "result"))
      {
 	  // Extract the query 
-	  Element* query = iq.findElement("query");
+	  const Element* query = iq.findElement("query");
 	  if (query != NULL)
 	  {      
 	       // Re load the agent list
@@ -244,9 +237,9 @@ void AgentBrowser::handle_agents_IQ(const Element& iq)
 
 void AgentBrowser::refresh()
 {
+     clear();
+
      _ctreeAgent->freeze();
-     // Clear the tree
-     _ctreeAgent->clear();	       
      // Render the agent list
      for(AgentList::iterator it = _agent_list.begin(); it != _agent_list.end(); it++)
 	  render_agent(*it, NULL);
@@ -353,13 +346,22 @@ void AgentBrowserDlg::on_close_clicked()
      close();
 }
 
-void AgentBrowserDlg::on_agent_selected(bool selected, bool registerable, bool searchable, bool gccapable, bool agents, bool transport)
+void AgentBrowserDlg::on_agent_selected(jabberoo::Agent* agent)
 {
      // Set the sensitivity of the appropriate buttons
-     _infoBtn->set_sensitive(selected);
-     _registerBtn->set_sensitive(registerable);
-     _searchBtn->set_sensitive(searchable);
-     _agent = _browser->get_current_agent();
+    if (agent)
+    {
+        _infoBtn->set_sensitive(true);
+        _registerBtn->set_sensitive(agent->isRegisterable());
+        _searchBtn->set_sensitive(agent->isSearchable());
+    }
+    else
+    {
+        _infoBtn->set_sensitive(false);
+        _registerBtn->set_sensitive(false);
+        _searchBtn->set_sensitive(false);
+    }
+    _agent = agent;
 }
 
 gint AgentBrowserDlg::on_key_pressed(GdkEventKey* e)
@@ -388,7 +390,7 @@ void AgentBrowserDlg::handle_agents_IQ_s(const Element& iq)
      if (iq.cmpAttrib("type", "result"))
      {
 	  // Extract the query 
-	  Element* query = iq.findElement("query");
+	  const Element* query = iq.findElement("query");
 	  if (query != NULL)
 		{
 	       // Create a new browser, passing a new AgentList
@@ -520,10 +522,10 @@ void AgentRegisterDruid::handle_request_reply(const Element& iq)
      if (iq.cmpAttrib("type", "result"))
      {
           // Extract the query 
-          Element* query = iq.findElement("query");
+          const Element* query = iq.findElement("query");
 
           if (_init_interface) {
-               _key = query->findElement_getCDATA("key");
+               _key = query->getChildCData("key");
 	  } else {
                Element::const_iterator it = query->begin();
 
@@ -657,7 +659,7 @@ void AgentRegisterDruid::on_register_reply(const Element& iq)
 {
      if (!iq.cmpAttrib("type", "result"))
      {
-          Element* error = iq.findElement("error");
+          const Element* error = iq.findElement("error");
 	  string message;
 	  _RDruid_registered->set_title(_("Registration Failed"));
 	  getFrame("RegDruid_Success_frame")->hide();
@@ -739,10 +741,10 @@ void AgentRegisterDruid::handle_unregister_reply(const Element& iq)
      if (iq.cmpAttrib("type", "result"))
      {
           // Extract the query 
-          Element* query = iq.findElement("query");
+          const Element* query = iq.findElement("query");
 	  string key;
 	  if (query)
-	       key = query->findElement_getCDATA("key");
+	       key = query->getChildCData("key");
 
 	  string id = G_App->getSession().getNextID();
 
@@ -751,10 +753,10 @@ void AgentRegisterDruid::handle_unregister_reply(const Element& iq)
 	  niq.setTo(iq.getAttrib("from"));
 	  niq.getBaseElement().putAttrib("type", "set");
 
-	  query = niq.getBaseElement().addElement("query");
-	  query->putAttrib("xmlns", "jabber:iq:register");
-	  query->addElement("key", key);
-	  query->addElement("remove");
+	  Element* newQuery = niq.getBaseElement().addElement("query");
+	  newQuery->putAttrib("xmlns", "jabber:iq:register");
+	  newQuery->addElement("key", key);
+	  newQuery->addElement("remove");
 	  G_App->getSession() << niq;
      }
 }
