@@ -76,9 +76,12 @@ ConfigManager::ConfigManager(const char* name)
      dates.format = getStrValue("Dates/Format=%c"); // julian likes %d %b %Y %H:%M:%S
      dates.gcformat = getStrValue("Dates/GCFormat=[%T] ");
 
-     docklet.show = getBoolValue("Docklet/Show=true");
+     docklet.show = getBoolValue("Docklet/Show=false");
+
+     filetransfer.port = getIntValue("FileTransfer/Port=0");
 
      gpg.enabled = getBoolValue("GPG/Enabled=false");
+     gpg.keyserver = getStrValue("GPG/keyserver=pgpkeys.mit.edu");
      gpg.secretkeyid = getStrValue("GPG/SecretKeyID=");
      for (i = 0; ; i++)
      {
@@ -166,6 +169,8 @@ ConfigManager::ConfigManager(const char* name)
      server.ssl = getBoolValue("Server/SSL=false");
      server.username = getStrValue("Server/Username=");
 
+     speech.usefestival = getBoolValue("Speech/UseFestival=false");
+
      spelling.check = getBoolValue("Spelling/Check=true");
      spelling.lang  = getStrValue("Spelling/Lang=");
 
@@ -183,6 +188,8 @@ ConfigManager::ConfigManager(const char* name)
      status.status = getStrValue("Status/Status=Online");
      status.type = getIntValue("Status/Type=2");
 
+     statusicon.show = getBoolValue("StatusIcon/Show=true");
+
      toolbars.menubar = getBoolValue("Toolbars/Menubar=true");
      toolbars.presence = getBoolValue("Toolbars/Presence=true");
      toolbars.status = getBoolValue("Toolbars/Status=false");
@@ -363,7 +370,10 @@ void ConfigManager::sync()
 
      putValue("Docklet/Show", docklet.show);
 
+     putValue("FileTransfer/Port", filetransfer.port);
+
      putValue("GPG/Enabled", gpg.enabled);
+     putValue("GPG/keyserver", gpg.keyserver);
      putValue("GPG/SecretKeyID", gpg.secretkeyid);
      i = 0;
      for (map<string, string>::iterator it = gpg.keymap.begin(); it != gpg.keymap.end(); it++)
@@ -429,7 +439,10 @@ void ConfigManager::sync()
      putValue("Server/AutoReconnect", server.autoreconnect);
      putValue("Server/Plaintext", server.plaintext);
      putValue("Server/LastUpdate", server.lastupdate);
-     putPrivValue("Server/Password", server.password);
+     if (server.savepassword)
+	  putPrivValue("Server/Password", server.password);
+     else
+	  putPrivValue("Server/Password", "");
      putValue("Server/Port", server.port);
      putValue("Server/Resource", server.resource);
      putValue("Server/SavePassword", server.savepassword);
@@ -440,6 +453,8 @@ void ConfigManager::sync()
      putValue("Spelling/Check", spelling.check);
      putValue("Spelling/Lang", spelling.lang);
 
+     putValue("Speech/UseFestival", speech.usefestival);
+
      putValue("Spool/Dir", spool.dir);
 
      putValue("Status/Priority", status.priority);
@@ -448,6 +463,8 @@ void ConfigManager::sync()
      putValue("Status/Status", status.status);
      putValue("Status/Type", status.type);
 
+     putValue("StatusIcon/Show", statusicon.show);
+
      putValue("Toolbars/Menubar", toolbars.menubar);
      putValue("Toolbars/Presence", toolbars.presence);
      putValue("Toolbars/Status", toolbars.status);
