@@ -29,6 +29,7 @@
 #include "GabberGPG.hh"
 
 #include "gtkspell.h"
+#include "gtkurl.h"
 
 #include <libgnome/gnome-help.h>
 #include <libgnome/gnome-i18n.h>
@@ -177,6 +178,8 @@ void ContactInfoDlg::init()
      _gpgInfo     = GabberGPG::gpgNone;
      getWidget<Gtk::EventBox>("ContactInfo_GPG_evt")->button_press_event.connect(slot(this, &ContactInfoDlg::on_GPGInfo_press_event));
 
+     gtkurl_attach(getWidget<Gtk::Text>("ContactInfo_About_txt")->gtkobj());
+
      _pjid = manage(new PrettyJID(_jid, "", PrettyJID::dtJIDRes, 128, true));
      _pjid->show_pixmap(false);
      _pjid->changed.connect(slot(this, &ContactInfoDlg::on_PrettyJID_changed));
@@ -229,31 +232,32 @@ void ContactInfoDlg::parse_vcard(const Element& t)
      if (!t.empty())
      {
 	  string curtext;
-	  Element* vCard = t.findElement("vCard");
+	  const Element* vCard = t.findElement("vCard");
 	  if (vCard != NULL)
 	  {
-	       curtext = vCard->findElement_getCDATA("NICKNAME");
-	       if (_entNickname->get_text().empty())
+	       curtext = vCard->getChildCData("NICKNAME");
+	       string nickname = toUTF8(_entNickname, _entNickname->get_text());
+	       if (nickname.empty())
 		    _entNickname->set_text(fromUTF8(_entNickname, curtext));
 	       // Save the old nickname
 	       _oldnick = curtext;
 	       // Allow them to revert to the old nickname if there actually is one
-	       getButton("ContactInfo_Nickname_Default_btn")->set_sensitive(!_oldnick.empty());
+	       getButton("ContactInfo_Nickname_Default_btn")->set_sensitive(!_oldnick.empty() && nickname != _oldnick);
 
 	       set_info_label("ContactInfo_FullName_lbl",
 			      "ContactInfo_PersInfo_frm",
 			      "ContactInfo_UserInfo_lbl",
-			      vCard->findElement_getCDATA("FN"));
+			      vCard->getChildCData("FN"));
 	       set_info_label("ContactInfo_eMail_lbl",
 			      "ContactInfo_PersInfo_frm",
 			      "ContactInfo_UserInfo_lbl",		      
-			      vCard->findElement_getCDATA("EMAIL"));
+			      vCard->getChildCData("EMAIL"));
 	       set_info_label("ContactInfo_Birthday_lbl",
 			      "ContactInfo_Birthday_frm",
 			      "ContactInfo_About_lbl",
-			      vCard->findElement_getCDATA("BDAY"));
+			      vCard->getChildCData("BDAY"));
 
-	       curtext = vCard->findElement_getCDATA("URL");
+	       curtext = vCard->getChildCData("URL");
 	       if (!curtext.empty())
 	       {
 		    Gnome::HRef* hrefWeb = getWidget<Gnome::HRef>("ContactInfo_WebSite_href");
@@ -266,59 +270,59 @@ void ContactInfoDlg::parse_vcard(const Element& t)
 	       set_info_label("ContactInfo_HomePhone_lbl",
 			      "ContactInfo_PersInfo_frm",
 			      "ContactInfo_UserInfo_lbl",
-			      vCard->findElement_getCDATA("TEL"));
+			      vCard->getChildCData("TEL"));
 
-	       Element* ADR = vCard->findElement("ADR");
+	       const Element* ADR = vCard->findElement("ADR");
 	       if (ADR != NULL)
 	       {
 		    set_info_label("ContactInfo_Street_lbl",
 				   "ContactInfo_Location_frm",
 				   "ContactInfo_Location_lbl",
-				   ADR->findElement_getCDATA("STREET"));
+				   ADR->getChildCData("STREET"));
 		    set_info_label("ContactInfo_Extadd_lbl",
 				   "ContactInfo_Location_frm",
 				   "ContactInfo_Location_lbl",
-				   ADR->findElement_getCDATA("EXTADD"));
+				   ADR->getChildCData("EXTADD"));
 		    set_info_label("ContactInfo_City_lbl",
 				   "ContactInfo_Location_frm",
 				   "ContactInfo_Location_lbl",
-				   ADR->findElement_getCDATA("LOCALITY"));
+				   ADR->getChildCData("LOCALITY"));
 		    set_info_label("ContactInfo_State_lbl",
 				   "ContactInfo_Location_frm",
 				   "ContactInfo_Location_lbl",
-				   ADR->findElement_getCDATA("REGION"));
+				   ADR->getChildCData("REGION"));
 		    set_info_label("ContactInfo_PCode_lbl",
 				   "ContactInfo_Location_frm",
 				   "ContactInfo_Location_lbl",
-				   ADR->findElement_getCDATA("PCODE"));
+				   ADR->getChildCData("PCODE"));
 		    set_info_label("ContactInfo_Country_lbl",
 				   "ContactInfo_Location_frm",
 				   "ContactInfo_Location_lbl",
-				   ADR->findElement_getCDATA("COUNTRY"));
+				   ADR->getChildCData("COUNTRY"));
 	       }
-	       Element* ORG = vCard->findElement("ORG");
+	       const Element* ORG = vCard->findElement("ORG");
 	       if (ORG != NULL)
 	       {
 		    set_info_label("ContactInfo_OrgName_lbl",
 				   "ContactInfo_OrgDetails_frm",
 				   "ContactInfo_Org_lbl",
-				   ORG->findElement_getCDATA("ORGNAME"));
+				   ORG->getChildCData("ORGNAME"));
 		    set_info_label("ContactInfo_OrgUnit_lbl",
 				   "ContactInfo_OrgDetails_frm",
 				   "ContactInfo_Org_lbl",
-				   ORG->findElement_getCDATA("ORGUNIT"));
+				   ORG->getChildCData("ORGUNIT"));
 	       }
 	       set_info_label("ContactInfo_Title_lbl",
 			      "ContactInfo_OrgPersDetails_frm",
 			      "ContactInfo_Org_lbl",
-			      vCard->findElement_getCDATA("TITLE"));
+			      vCard->getChildCData("TITLE"));
 	       set_info_label("ContactInfo_Role_lbl",
 			      "ContactInfo_OrgPersDetails_frm",
 			      "ContactInfo_Org_lbl",
-			      vCard->findElement_getCDATA("ROLE"));
+			      vCard->getChildCData("ROLE"));
 
 	       // Description, aka About
-	       curtext = vCard->findElement_getCDATA("DESC");
+	       curtext = vCard->getChildCData("DESC");
 	       Gtk::Text* t = getWidget<Gtk::Text>("ContactInfo_About_txt");
 	       t->set_word_wrap(true);
 	       int i = 0;
@@ -326,6 +330,7 @@ void ContactInfoDlg::parse_vcard(const Element& t)
 	       {
 		    t->insert_text(fromUTF8(t, curtext).c_str(),
 				   fromUTF8(t, curtext).length(), &i);
+		    gtkurl_check_all(t->gtkobj());
 		    // Hrmm, this doesn't seem to scroll it back to the beginning
 		    t->set_point(0);
 		    getFrame("ContactInfo_About_frm")->show();
@@ -359,7 +364,7 @@ void ContactInfoDlg::parse_last(const Element& t)
      {
 	  int seconds;
 	  string lasttime;
-	  Element* query = t.findElement("query");
+	  const Element* query = t.findElement("query");
 	  if (query != NULL)
 	  {
 	       // Grab the idle or last logged out time
@@ -447,7 +452,7 @@ void ContactInfoDlg::parse_last(const Element& t)
 		    lasttime += " ago";
 	       lasttime += ")";
 
-	       string status = fromUTF8(_lblStatus, t.findElement_getCDATA("query")); // Get the status from iq:last
+	       string status = fromUTF8(_lblStatus, t.getChildCData("query")); // Get the status from iq:last
 	       if (_has_presence || status.empty())       // unless we already have status
 		    status = _lblStatus->get_text();
 
@@ -479,21 +484,21 @@ void ContactInfoDlg::parse_version(const Element& t)
 {
      if (!t.empty())
      {
-	  Element* query = t.findElement("query");
+	  const Element* query = t.findElement("query");
 	  if (query != NULL)
 	  {
 	       set_info_label("ContactInfo_Client_lbl",
 			      "ContactInfo_JabberClient_frm",
 			      "ContactInfo_ClientInfo_lbl",
-			      query->findElement_getCDATA("name"));
+			      query->getChildCData("name"));
 	       set_info_label("ContactInfo_ClientVersion_lbl",
 			      "ContactInfo_JabberClient_frm",
 			      "ContactInfo_ClientInfo_lbl",
-			      query->findElement_getCDATA("version"));
+			      query->getChildCData("version"));
 	       set_info_label("ContactInfo_OS_lbl",
 			      "ContactInfo_Computer_frm",
 			      "ContactInfo_ClientInfo_lbl",
-			      query->findElement_getCDATA("os"));
+			      query->getChildCData("os"));
 	  }
      }
 }
@@ -521,12 +526,13 @@ void ContactInfoDlg::parse_time(const Element& t)
      if (!t.empty())
      {
 	  string curtext;
-	  Element* query = t.findElement("query");
+	  const Element* query = t.findElement("query");
 	  if (query != NULL)
 	  {
-	       curtext = query->findElement_getCDATA("display");
-	       curtext += " ";
-	       curtext += query->findElement_getCDATA("tz");
+	       curtext = query->getChildCData("display");
+	       curtext += " "; // Don't duplicate the time zone:
+	       if(curtext.find(query->getChildCData("tz"))==string::npos)
+		       curtext += query->getChildCData("tz");
 	       set_info_label("ContactInfo_Time_lbl",
 			      "ContactInfo_Computer_frm",
 			      "ContactInfo_ClientInfo_lbl",
@@ -647,6 +653,8 @@ void ContactInfoDlg::on_nickname_changed()
 	  Gtk::Frame* f = getFrame("ContactInfo_Computer_frm");
 	  f->set_label(substitute(_("%s's Computer"), fromUTF8(f, nickname)));
      }
+     getButton("ContactInfo_OK_btn")->set_sensitive(!nickname.empty());
+     getButton("ContactInfo_Nickname_Default_btn")->set_sensitive(!_oldnick.empty() && _oldnick != nickname);
 }
 
 void ContactInfoDlg::on_PrettyJID_changed()
@@ -791,10 +799,10 @@ void MyContactInfoWin::manage_query(const string& jid)
 void MyContactInfoWin::get_key(const Element& t)
 {
      // Set the <key>
-     Element* query = t.findElement("query");
+     const Element* query = t.findElement("query");
      if (query)
      {
-	  _key = query->findElement_getCDATA("key");
+	  _key = query->getChildCData("key");
 	  _keyUsed = false;
      }
 }
@@ -824,31 +832,31 @@ void MyContactInfoWin::get_vCard(const Element& t)
      if (!t.empty())
      {
 	  string curtext;
-	  Element* vCard = t.findElement("vCard");
+	  const Element* vCard = t.findElement("vCard");
 	  if (vCard != NULL)
 	  {
 	       // Nickname
-	       curtext = vCard->findElement_getCDATA("NICKNAME");
+	       curtext = vCard->getChildCData("NICKNAME");
 	       _entNickname->set_text(fromUTF8(_entNickname, curtext));
 	       // Name
-	       Element* N = vCard->findElement("N");
+	       const Element* N = vCard->findElement("N");
 	       if (N != NULL)
 	       {
 		    // Family Name
-		    curtext = N->findElement_getCDATA("FAMILY");
+		    curtext = N->getChildCData("FAMILY");
 		    _entLastName->set_text(fromUTF8(_entLastName, curtext));
 		    // Given Name
-		    curtext = N->findElement_getCDATA("GIVEN");
+		    curtext = N->getChildCData("GIVEN");
 		    _entFirstName->set_text(fromUTF8(_entFirstName, curtext));
 	       }
 	       // Full Name
-	       curtext = vCard->findElement_getCDATA("FN");
+	       curtext = vCard->getChildCData("FN");
 	       _entFullName->set_text(fromUTF8(_entFullName, curtext));
 	       // Email
-	       curtext = vCard->findElement_getCDATA("EMAIL");
+	       curtext = vCard->getChildCData("EMAIL");
 	       _entEMail->set_text(fromUTF8(_entEMail, curtext));
 	       // Birthday
-	       curtext = vCard->findElement_getCDATA("BDAY");
+	       curtext = vCard->getChildCData("BDAY");
 	       // We need a way to parse out the birthday and set it if it fits the proper format
 	       // Should be in iso 8601 format YYYY-MM-DD (time, which I didn't look at)
 	       // Time isn't important for birthday
@@ -868,65 +876,62 @@ void MyContactInfoWin::get_vCard(const Element& t)
 	       }
 	       getWidget<Gnome::DateEdit>("MyContactInfo_Personal_dateedit")->set_time(bday);
 	       // Web Site
-	       curtext = vCard->findElement_getCDATA("URL");
+	       curtext = vCard->getChildCData("URL");
 	       Gtk::Entry* e = getEntry("MyContactInfo_Personal_WebSite_txt");
 	       e->set_text(fromUTF8(e, curtext));
 	       // Phone Number
-	       curtext = vCard->findElement_getCDATA("TEL");
+	       curtext = vCard->getChildCData("TEL");
 	       e = getEntry("MyContactInfo_Personal_HomePhone_txt");
 	       e->set_text(fromUTF8(e, curtext));
 	       // Address
-	       Element* ADR = vCard->findElement("ADR");
+	       const Element* ADR = vCard->findElement("ADR");
 	       if (ADR != NULL)
 	       {
-		    curtext = ADR->findElement_getCDATA("STREET");
+		    curtext = ADR->getChildCData("STREET");
 		    e = getEntry("MyContactInfo_Address_Street_txt");
 		    e->set_text(fromUTF8(e, curtext));
-		    curtext = ADR->findElement_getCDATA("EXTADD");
+		    curtext = ADR->getChildCData("EXTADD");
 		    e = getEntry("MyContactInfo_Address_Extadd_txt");
 		    e->set_text(fromUTF8(e, curtext));
-		    curtext = ADR->findElement_getCDATA("LOCALITY");
+		    curtext = ADR->getChildCData("LOCALITY");
 		    e = getEntry("MyContactInfo_Address_City_txt");
 		    e->set_text(fromUTF8(e, curtext));
-		    curtext = ADR->findElement_getCDATA("REGION");
+		    curtext = ADR->getChildCData("REGION");
 		    e = getEntry("MyContactInfo_Address_State_txt");
 		    e->set_text(fromUTF8(e, curtext));
-		    curtext = ADR->findElement_getCDATA("PCODE");
+		    curtext = ADR->getChildCData("PCODE");
 		    e = getEntry("MyContactInfo_Address_PCode_txt");
 		    e->set_text(fromUTF8(e, curtext));
-		    curtext = ADR->findElement_getCDATA("COUNTRY");
+		    curtext = ADR->getChildCData("COUNTRY");
 		    e = getEntry("MyContactInfo_Address_Country_txt");
 		    e->set_text(fromUTF8(e, curtext));
 	       }
-	       Element* ORG = vCard->findElement("ORG");
+	       const Element* ORG = vCard->findElement("ORG");
 	       if (ORG != NULL)
 	       {
-		    curtext = ORG->findElement_getCDATA("ORGNAME");
+		    curtext = ORG->getChildCData("ORGNAME");
 		    e = getEntry("MyContactInfo_OrgName_txt");
 		    e->set_text(fromUTF8(e, curtext));
-		    curtext = ORG->findElement_getCDATA("ORGUNIT");
+		    curtext = ORG->getChildCData("ORGUNIT");
 		    e = getEntry("MyContactInfo_OrgUnit_txt");
 		    e->set_text(fromUTF8(e, curtext));
 	       }
-	       curtext = vCard->findElement_getCDATA("TITLE");
+	       curtext = vCard->getChildCData("TITLE");
 	       e = getEntry("MyContactInfo_Title_txt");
 	       e->set_text(fromUTF8(e, curtext));
-	       curtext = vCard->findElement_getCDATA("ROLE");
+	       curtext = vCard->getChildCData("ROLE");
 	       e = getEntry("MyContactInfo_Role_txt");
 	       e->set_text(fromUTF8(e, curtext));
 	       // Description, aka About
-	       curtext = vCard->findElement_getCDATA("DESC");
+	       curtext = vCard->getChildCData("DESC");
 	       Gtk::Text* t = getWidget<Gtk::Text>("MyContactInfo_About_txt");
 	       t->set_word_wrap(true);
-	       if (curtext.length() > 0)
-	       {
-		    // Clear the default text
-		    t->delete_text(0, -1);
-		    int i = 0;
-		    // Insert the text we found
-		    t->insert_text(fromUTF8(t, curtext).c_str(),
-				   fromUTF8(t, curtext).length(), &i);
-	       }
+	       // Clear the default text
+	       t->delete_text(0, -1);
+	       int i = 0;
+	       // Insert the text we found
+	       t->insert_text(fromUTF8(t, curtext).c_str(),
+			      fromUTF8(t, curtext).length(), &i);
 	       // Hrmm, this doesn't seem to scroll it back to the beginning
 	       t->set_point(0);
 	  }
@@ -1072,7 +1077,7 @@ void MyContactInfoWin::saveconfig()
 
 void MyContactInfoWin::changed()
 {
-     // Nadda for now
+     getButton("MyContactInfo_OK_btn")->set_sensitive(!_entNickname->get_text().empty());
 }
 
 void MyContactInfoWin::on_name_changed()
@@ -1254,14 +1259,16 @@ AgentInfoDlg::AgentInfoDlg(const string& jid)
 
 void AgentInfoDlg::on_ok_clicked()
 {
-     try {
-	  Roster::Item item(G_App->getSession().roster()[_jid]);
-          item.setNickname(toUTF8(_entNickname, _entNickname->get_text()));
-	  G_App->getSession().roster() << item;
-     } catch (Roster::XCP_InvalidJID& e) {
-	  cerr << "Nickname change unsuccessful" << endl;
+     if (_entNickname->is_visible())
+     {
+	  try {
+	       Roster::Item item(G_App->getSession().roster()[_jid]);
+	       item.setNickname(toUTF8(_entNickname, _entNickname->get_text()));
+	       G_App->getSession().roster() << item;
+	  } catch (Roster::XCP_InvalidJID& e) {
+	       cerr << "Nickname change unsuccessful" << endl;
+	  }
      }
-
      close();
 }
 
