@@ -22,6 +22,7 @@
  * Author Thomas "temas" Muldowney <temas@jabber.org>
  */
 
+#include "ghttp.c"
 #include "GabberConfig.hh" // for _()
 
 #include "FTInterface.hh"
@@ -37,18 +38,12 @@
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <netdb.h>
-
-#include <fstream.h>
 
 using namespace jabberoo;
 using namespace GabberUtil;
 
-const guint LISTEN_PORT = 0;
-
 static std::string printBytes(float bytes)
 {
     char string_buffer[32];
@@ -107,6 +102,7 @@ void FTRecvView::render(const judo::Element& baseElement)
     _clistFiles->freeze();
     Element* child;
     Gtk::Label* l = getLabel("FTRecv_Description_lbl");
+    int goodURIs = 0; 
     for (; it != baseElement.end(); it++)
     {
         child = static_cast<Element*>(*it);
@@ -118,13 +114,30 @@ void FTRecvView::render(const judo::Element& baseElement)
             (child->getAttrib("xmlns") == "jabber:iq:oob"))
         {
             const char* data[2];
-            data[0] = fromUTF8(_clistFiles, child->findElement_getCDATA("url")).c_str();
-            l->set_text(fromUTF8(l, child->findElement_getCDATA("desc")));
-
-            _clistFiles->rows().push_back(data);
+            string uri = fromUTF8(_clistFiles, child->getChildCData("url"));
+            data[0] = uri.c_str();
+            if ( ghttp_uri_validate((char*)data[0]) != -1 )
+            { 
+                ++goodURIs; // This is a good URI.  Count it.
+                // XXX something else needs to be done with "desc" CDATA.
+                // If there are multiple files, we'll only see desc for the
+                // last one:
+                l->set_text(fromUTF8(l, child->getChildCData("desc")));
+                _clistFiles->rows().push_back(data);
+            }
+            else  // Mark the URI as invalid (but still display it).
+            {
+                data[0] = ("INVALID URL: " + uri).c_str();
+                _clistFiles->rows().push_back(data);
+            }
         }
     }
     _clistFiles->thaw();
+    if (goodURIs < 1)
+    {  // If all of the URIs are invalid, don't allow the user to attempt a download.
+        getButton("FTRecv_Accept_btn")->set_sensitive(false);
+        l->set(substitute(_("%s tried to offer files, but all the URLs sent are invalid."), fromUTF8(PrettyJID(_jid, "", PrettyJID::dtNickRes).get_nickname())));
+    }
 }
 
 void FTRecvView::on_Remove_clicked()
@@ -138,7 +151,7 @@ void FTRecvView::on_Remove_clicked()
 
 void FTRecvView::on_Accept_clicked()
 {
-    // XXX Do stuff to start the get
+    // XXX Do stuff to start the get -- make sure calls to ghttp_uri_validate() are involved.
     std::string path = _gfeSaveTo->get_full_path(false);
     if (path.empty())
     {
@@ -152,6 +165,10 @@ void FTRecvView::on_Accept_clicked()
     {
        Gtk::CList::Row& row = *(it);
        std::string url = toUTF8(_clistFiles, row[0].get_text());
+
+       // FIXME: Build a string list of URLs which pass ghttp_uri_validate()
+       // FIXME: make FTTransferDlg accept this string list and go through and download them all
+       manage(new FTTransferDlg(_jid, url, path));
     }
     close();
 }
@@ -304,7 +321,7 @@ FTTransferDlg::FTTransferDlg(const std::string& jid, const std::string& file,
     getButton("FTTransfer_Stop_btn")->clicked.connect(slot(this, &FTTransferDlg::on_Cancel_clicked));
 
     // Listen on all hosts for a random port
-    _Transmitter->listen("", LISTEN_PORT);
+    _Transmitter->listen("", G_App->getCfg().filetransfer.port);
 
     char port[8];
     char hostname[1024];
@@ -460,7 +477,7 @@ void FTTransferDlg::on_Cancel_clicked()
 void FTTransferDlg::on_transmitter_connected()
 {
      _lblStatus->set_text(_("Connected, requesting file..."));
-     std::string hdr("GET " + _path + " HTTP/1.0\n\n");
+     std::string hdr("GET " + _path + " HTTP/1.0\r\n\r\n");
      _Transmitter->send(hdr.c_str());
 }
 
@@ -558,13 +575,20 @@ void FTTransferDlg::handleSend(const char* data, int sz)
 {
      if (!_do_data)
      {
+          // Make sure we are handling a GET call
+          if (strncasecmp(data, "GET", 3) != 0)
+          {
+              _Transmitter->send("HTTP/1.0 400 Bad Request\r\n");
+              _Transmitter->disconnect();
+          }
+
 	  _lblStatus->set_text(_("Sending..."));
 	  _Transmitter->send("HTTP/1.0 200 OK\r\n");
 	  char cl_buf[10];
 	  snprintf(cl_buf, 10, "%d", _content_length);
 	  _Transmitter->send("Content-Length: ");
 	  _Transmitter->send(cl_buf);
-	  _Transmitter->send("\r\n");
+      _Transmitter->send("\r\n");
 	  _Transmitter->send("Content-Type: text/plain\r\n\r\n");
 	  
 	  _do_data = true;
