@@ -88,7 +88,7 @@ ActionView::ActionView(Filter::ActionList& alist, Filter::ActionList::iterator i
      // Create parameters entry -- 256 chars max
      _pentry = manage(new Gtk::Entry(256));
      _pentry->changed.connect(slot(this, &ActionView::on_ParamChange));
-     _pentry->set_text(_action->param());
+     _pentry->set_text(fromUTF8(_action->param()));
      _pentry->set_usize(120, 0);
      add_show(this, _pentry, true, true, 4);
      if (it->requires_param())
@@ -118,7 +118,7 @@ ActionView::ActionView(Filter::ActionList& alist, Filter::ActionList::iterator i
 void ActionView::on_ParamChange()
 {
      // Update the underlying action
-     *_action << _pentry->get_text();
+     *_action << toUTF8(_pentry->get_text());
 }
 
 void ActionView::on_ActionChange(int index)
@@ -202,7 +202,7 @@ ConditionView::ConditionView(Filter::ConditionList& clist, Filter::ConditionList
      // Create parameters entry -- 256 chars max
      _pentry = manage(new Gtk::Entry(256));
      _pentry->changed.connect(slot(this, &ConditionView::on_ParamChange));
-     _pentry->set_text(_condition->param());
+     _pentry->set_text(fromUTF8(_condition->param()));
      _pentry->set_usize(120,0);
      add_show(this, _pentry, true, true, 4);
      if (it->requires_param())
@@ -232,7 +232,7 @@ ConditionView::ConditionView(Filter::ConditionList& clist, Filter::ConditionList
 void ConditionView::on_ParamChange()
 {
      // Update underlying condition
-     *_condition << _pentry->get_text();
+     *_condition << toUTF8(_pentry->get_text());
 }
 
 void ConditionView::on_ConditionChange(int index)
@@ -416,7 +416,7 @@ void FilterListView::handleFilterIQ(const Element& iq)
      if (iq.cmpAttrib("type", "result"))
      {
 	  // Extract the query 
-	  Element* query = iq.findElement("query");
+	  const Element* query = iq.findElement("query");
 	  if (query != NULL)
 		{
 	       // Create a new view, passing a FilterList constructor
@@ -454,7 +454,7 @@ void FilterListView::refresh()
      for (FilterList::iterator it = _flist.begin(); it != _flist.end(); it++)
      {
 	  // Insert a new row in the filter list
-	  const char* rowdata[1] = { it->Name().c_str() };
+	  const char* rowdata[1] = { fromUTF8(it->Name()).c_str() };
 	  int row = _lstFilters->append_row(rowdata);
 	  // Store the corresponding filter address in the last element's data section	  
 	  Filter* fp = &(*it);
@@ -493,7 +493,7 @@ FilterView::FilterView(Filter& f)
      getButton("Filter_Cancel_btn")->clicked.connect(slot(this, &FilterView::on_CancelClick));
 
      // Load filter name
-     _entFilterName->set_text(f.Name());
+     _entFilterName->set_text(fromUTF8(f.Name()));
 
      // Walk the filter's conditions and create a view for each
      for (Filter::ConditionList::iterator it = _filter.Conditions().begin(); it != _filter.Conditions().end(); it++)
@@ -545,7 +545,7 @@ void FilterView::on_NewConditionClick()
 void FilterView::on_FilterNameChange()
 {
      // Update filter name
-     _filter.setName(_entFilterName->get_text());
+     _filter.setName(toUTF8(_entFilterName->get_text()));
 }
 
 void FilterView::on_OKClick()
