@@ -35,6 +35,8 @@
 
 #include "GPGInterface.hh"
 
+#include "GabberApp.hh" // For G_App->getCfg().gpg.keyserver, nothing else
+
 #ifndef GPG_DEBUG
 #define GPG_DEBUG false
 #endif
@@ -686,7 +688,11 @@ bool GPGInterface::import_key(KeyInfo& info, istream* key)
 bool GPGInterface::key_info_from_key(KeyInfo& info, istream* key)
 {
      list<string> args;
+#ifdef HAVE_STD_SSTREAM
+     stringstream output;
+#else
      strstream output;
+#endif
 
      _error = errNone;
      _input = key;
@@ -753,7 +759,11 @@ bool GPGInterface::key_info_from_key(KeyInfo& info, istream* key)
 bool GPGInterface::key_info_from_id(KeyInfo& info, const string& keyid)
 {
      list<string> args;
+#ifdef HAVE_STD_SSTREAM
+     stringstream output;
+#else
      strstream output;
+#endif
 
      _error = errNone;
      _input = NULL;
@@ -874,8 +884,11 @@ bool GPGInterface::export_key(ostream* dest, const string& keyid)
 bool GPGInterface::get_keys(list<KeyInfo>& keys)
 {
      list<string> args;
-     // This would be much better as a stringstream but I don't have them on my machine
+#ifdef HAVE_STD_SSTREAM
+     stringstream output;
+#else
      strstream output;
+#endif
 
      _error = errNone;
      _input = NULL;
@@ -945,7 +958,11 @@ bool GPGInterface::get_keys(list<KeyInfo>& keys)
 bool GPGInterface::get_secret_keys(list<KeyInfo>& keys)
 {
      list<string> args;
+#ifdef HAVE_STD_SSTREAM
+     stringstream output;
+#else
      strstream output;
+#endif
 
      _error = errNone;
      _input = NULL;
@@ -1066,7 +1083,7 @@ bool GPGInterface::execute(list<string> func_args, bool use_shm)
      }
      args.push_back("--utf8-strings");
      args.push_back("--keyserver");
-     args.push_back(GPG_KEYSERVER);
+     args.push_back(G_App->getCfg().gpg.keyserver);
 
      if (_armor)
           args.push_back("--armor");
