@@ -28,6 +28,7 @@
 #include "WelcomeDruid.hh"
 // Dialogs
 #include "ErrorManager.hh"
+#include "EventManager.hh"
 #include "MessageManager.hh"
 #include "MessageViews.hh"
 #include "PrefsInterface.hh"
@@ -106,6 +107,7 @@ GabberApp::GabberApp(int argc, char** argv)
 				     _Cfg->logs.html);
      _MessageManager = new MessageManager(_Cfg->spool.dir);
      _ErrorManager = new ErrorManager();
+     _EventManager = new EventManager();
      _GabberGPG = new GabberGPG;
      _GabberGPG->set_armor(true);
     
@@ -115,6 +117,7 @@ GabberApp::GabberApp(int argc, char** argv)
      Message::setDateTimeFormat(_Cfg->dates.format);
 
      // Debugging signals     
+     _Session->evtXMLParserError.connect(slot(this, &GabberApp::XML_parser_error));
      _Session->evtTransmitXML.connect(slot(this, &GabberApp::transmit_XML));
      _Session->evtRecvXML.connect(slot(this, &GabberApp::recv_XML));
      _Session->evtTransmitPacket.connect(slot(this, &GabberApp::transmit_packet));
@@ -128,12 +131,14 @@ GabberApp::GabberApp(int argc, char** argv)
      _Session->evtUnknownPacket.connect(slot(this, &GabberApp::on_session_unknown_packet));
      _Session->evtDisconnected.connect(slot(this, &GabberApp::on_session_disconnected));
      _Session->evtOnVersion.connect(slot(this, &GabberApp::on_session_version));
+     _Session->evtOnTime.connect(slot(this, &GabberApp::on_session_time));
      _Session->evtPresence.connect(slot(this, &GabberApp::on_session_presence));
      _Session->evtMyPresence.connect(slot(this, &GabberApp::on_session_my_presence));
 
      // Connect up transmitter signal
      _Transmitter->evtConnected.connect(slot(this, &GabberApp::on_transmitter_connected));
      _Transmitter->evtDisconnected.connect(slot(this, &GabberApp::on_transmitter_disconnected));
+     _Transmitter->evtReconnect.connect(slot(this, &GabberApp::on_transmitter_reconnect));
      _Transmitter->evtError.connect(slot(this, &GabberApp::on_transmitter_error));
      _Transmitter->evtDataAvailable.connect(slot(_Session, &Session::push));
 
@@ -197,10 +202,17 @@ void GabberApp::init_spellcheck()
      if (!gtkspell_running())
      {
 	  // Start up gtkspell for spell checking
-//	  if (!_Cfg->spelling.lang.empty())
-
 	  char* spellcmd[] = { "aspell", "--sug-mode=fast",
 			       "pipe", NULL };
+// 	  if (!_Cfg->spelling.lang.empty())
+// 	  {
+// 	       char* langtag = g_strdup_printf("--language-tag=%s", _Cfg->spelling.lang.c_str());
+// 	       realloc(spellcmd, sizeof(langtag)*4);
+// 	       spellcmd[2] = langtag;
+// 	       spellcmd[3] = "pipe";
+// 	       spellcmd[4] = NULL;
+// 	       g_free(langtag);
+// 	  }
 	  if (gtkspell_start(NULL, spellcmd) < 0)
 	  {
 	       char* spellcmd2[] = { "ispell", "-a", NULL };
@@ -221,6 +233,34 @@ void GabberApp::stop_spellcheck()
      }
 }
 
+void GabberApp::XML_parser_error(int error_code, const string& error_msg)
+{
+     // Give them cake
+     Gnome::Dialog* d;
+     string errmsg = _("An error occurred while trying to parse the received XML.\n"
+		       "You have been disconnected.");
+     if (_Cfg->server.autoreconnect)
+	  errmsg += _("\nAttempting to reconnect...");
+     errmsg += string("\nError: ") + error_msg;
+
+     d = manage(Gnome::Dialogs::warning(errmsg));
+     d->set_modal(true);
+     main_dialog(d);
+
+     // Let them eat cake (try to disconnect as gracefully as possible)
+     if (G_Win)
+	  G_Win->display_status(Presence::stOffline, _("Logged out"), 0, true, Presence::ptUnavailable);
+     if (!_Session->disconnect())
+	  _Transmitter->disconnect();
+
+     if (_Cfg->server.autoreconnect)
+     {
+	  // Try to give them more cake (reconnect)
+	  init_win();
+	  login();
+     }
+}
+
 void GabberApp::transmit_XML(const char* XML)
 {
      cerr << jutil::getTimeStamp() << "<<< " << XML << endl;
@@ -256,6 +296,7 @@ GabberApp::~GabberApp()
      delete _Logger;
      delete _MessageManager;
      delete _ErrorManager;
+     delete _EventManager;
      delete _GabberGPG;
      delete _AutoAway;
 
@@ -401,14 +442,7 @@ void GabberApp::on_session_message(const Message& m)
 	  _ErrorManager->add(m);
 	  return;
      }
-     
-     // If there appear to be events in this message
-     Element* x = m.findX("jabber:x:event");
-     if (x != NULL && !m.getBody().empty() && x->findElement("delivered") != NULL)
-     {
-	  // delivered event was requested and this message has been delivered
-	  *_Session << m.delivered();
-     }
+
      if ( (m.findX("jabber:x:encrypted")) != NULL)
 	     encrypted = true;
      // If logging is enabled and the message body isn't empty
@@ -423,7 +457,7 @@ void GabberApp::on_session_message(const Message& m)
 
 void GabberApp::on_session_iq(const Element& t)
 {
-    Element* q = t.findElement("query");
+    const Element* q = t.findElement("query");
 
     // Catch the odd case of an IQ not having a <query> tag..
     // It's not really so odd, temas, there is browsing ;)
@@ -530,13 +564,13 @@ void GabberApp::on_session_presence_request(const Presence& p)
 
 void GabberApp::on_agent_reply(const Element& iq)
 {
-     Element* query = iq.findElement("query");
+     const Element* query = iq.findElement("query");
      if (iq.cmpAttrib("type", "result")) 
      {
           string jid = iq.getAttrib("jid");
 	  if (query != NULL)
 	  {
-	       string agentname = query->findElement_getCDATA("name");
+	       string agentname = query->getChildCData("name");
 	       // Rename the agent to its actual name
 	       if (!agentname.empty())
 		    _Session->roster() << Roster::Item(_current_agent_jid, agentname);
@@ -546,12 +580,12 @@ void GabberApp::on_agent_reply(const Element& iq)
 
 void GabberApp::on_agents_reply(const Element& iq)
 {
-     Element* query = iq.findElement("query");
+     const Element* query = iq.findElement("query");
      if (iq.cmpAttrib("type", "result") && query != NULL)
      {
 	  // Walk the children
 	  Element::const_iterator it = query->begin();
-	  for (; it != query->end(); it++)
+	  for (; it != query->end(); ++it)
           {
 	       if ((*it)->getType() != Node::ntElement)
 		    continue;
@@ -563,7 +597,7 @@ void GabberApp::on_agents_reply(const Element& iq)
                string jid = agent.getAttrib("jid");
 	       if (jid != _current_agent_host)
 		    continue;
-	       string agentname = agent.findElement_getCDATA("name");
+	       string agentname = agent.getChildCData("name");
 	       // Rename the agent to its actual name
 	       if (!agentname.empty())
 		    _Session->roster() << Roster::Item(_current_agent_jid, agentname);
@@ -593,6 +627,12 @@ void GabberApp::on_session_version(string& name, string& version, string& os)
      os = string(osinfo.sysname) + " " + string(osinfo.machine);
 }
 
+void GabberApp::on_session_time(string& UTF8Time, string& UTF8TimeZone)
+{
+     UTF8Time = toUTF8(UTF8Time);
+     UTF8TimeZone = toUTF8 (UTF8TimeZone);
+}
+
 // -------------------------------------
 //
 // Transmitter event handlers
@@ -641,6 +681,12 @@ void GabberApp::on_transmitter_disconnected()
      }
 }
 
+void GabberApp::on_transmitter_reconnect()
+{
+     if (G_Win)
+	  G_Win->reconnecting();
+}
+
 /*void GabberApp::on_transmitter_error(TCPTransmitter::Error e)*/
 void GabberApp::on_transmitter_error(const string & emsg)
 {
@@ -683,7 +729,7 @@ void GabberApp::on_session_presence(const jabberoo::Presence& p, const jabberoo:
 	  cerr << "got unavailable presence from transport" << endl;
           Roster::iterator it = _Session->roster().begin();
           string transport = JID::getHost(p.getFrom());
-          for ( ; it != _Session->roster().end(); it++)
+          for ( ; it != _Session->roster().end(); ++it)
           {
                // Check if the jid is from the transport but skip over the transport itself
                if ((transport == JID::getHost(it->getJID())) && (p.getFrom() != it->getJID()))
@@ -724,7 +770,7 @@ void GabberApp::on_session_presence(const jabberoo::Presence& p, const jabberoo:
                     cerr << "Couldn't sign status" << endl;
 	  }
 
-          for ( ; it != _Session->roster().end(); it++)
+          for ( ; it != _Session->roster().end(); ++it)
 	  {
 	       if ((transport == JID::getHost(it->getJID())) && (p.getFrom() != it->getJID()))
 	       {
