@@ -62,9 +62,15 @@ GabberGPG::~GabberGPG()
 
 GPGInterface::Error GabberGPG::sign(GPGInterface::SignType type, const string& source, string& dest)
 {
-     // stringstreams are so much better but but libstdc++ doesn't seem to have them
+     // stringstreams are so much better but earlier versions
+     // of libstdc++ don't have them
+#ifdef HAVE_STD_SSTREAM
+     istringstream istr(source.c_str());
+     stringstream ostr;
+#else
      istrstream istr(source.c_str());
      strstream ostr;
+#endif
 
      // Initialize the GPGInterface
      GPGInterface signer(G_App->getCfg().gpg.secretkeyid, _passphrase, _armor);
@@ -82,8 +88,13 @@ GPGInterface::Error GabberGPG::sign(GPGInterface::SignType type, const string& s
 
 GPGInterface::Error GabberGPG::encrypt(const string& recipient, const string& source, string& dest, bool sign)
 {
+#ifdef HAVE_STD_SSTREAM
+     istringstream istr(source.c_str());
+     stringstream ostr;
+#else
      istrstream istr(source.c_str());
      strstream ostr;
+#endif
 
      // Initialise the GPGInterface to encrypt
      GPGInterface encrypter(G_App->getCfg().gpg.secretkeyid, _passphrase, _armor);
@@ -100,8 +111,13 @@ GPGInterface::Error GabberGPG::encrypt(const string& recipient, const string& so
 
 GPGInterface::Error GabberGPG::decrypt(GPGInterface::DecryptInfo& decrypt, const string& source, string& dest)
 {
+#ifdef HAVE_STD_SSTREAM
+     stringstream istr;
+     ostringstream ostr;
+#else
      strstream istr;
      ostrstream ostr;
+#endif
 
      GPGInterface decrypter(G_App->getCfg().gpg.secretkeyid, _passphrase, _armor);
      decrypter.gpgNeedPassphrase.connect(slot(this, &GabberGPG::get_passphrase));
@@ -122,7 +138,11 @@ GPGInterface::Error GabberGPG::decrypt(GPGInterface::DecryptInfo& decrypt, const
 
 GPGInterface::Error GabberGPG::verify(GPGInterface::SigInfo& sig, const string& signature, const string& message)
 {
+#ifdef HAVE_STD_SSTREAM
+     stringstream istr;
+#else
      strstream istr;
+#endif
      char filename[L_tmpnam];
      
      if (tmpnam(filename) == NULL)
@@ -159,7 +179,11 @@ GPGInterface::Error GabberGPG::verify(GPGInterface::SigInfo& sig, const string&
 // Import a Key from a string
 GPGInterface::Error GabberGPG::import_key(GPGInterface::KeyInfo& info, const string& key)
 {
+#ifdef HAVE_STD_SSTREAM
+     istringstream istr(key.c_str());
+#else
      istrstream istr(key.c_str());
+#endif
 
      GPGInterface importer;
 
@@ -177,7 +201,11 @@ GPGInterface::Error GabberGPG::export_key(string& dest)
 // export the key with the given keyid.  If none is gived, the public key for the local keyid is exported to dest
 GPGInterface::Error GabberGPG::export_key(string& dest, const string& keyid)
 {
+#ifdef HAVE_STD_SSTREAM
+     ostringstream ostr;
+#else
      ostrstream ostr;
+#endif
 
      GPGInterface exporter;
      exporter.set_armor(_armor);
@@ -192,7 +220,11 @@ GPGInterface::Error GabberGPG::export_key(string& dest, const string& keyid)
 // Get key information from the key given in the string key
 GPGInterface::Error GabberGPG::key_info_from_key(GPGInterface::KeyInfo& info, const string& key)
 {
-     istrstream istr(key.c_str());;
+#ifdef HAVE_STD_SSTREAM
+     istringstream istr(key.c_str());
+#else
+     istrstream istr(key.c_str());
+#endif
 
      GPGInterface infoer;
 
@@ -367,8 +399,14 @@ void GabberGPG::sigpipe_handler(int signo)
 
 GPGInterface::Error GabberGPG::verify_clear(GPGInterface::SigInfo& sig, const string& signature, string& message)
 {
+#ifdef HAVE_STD_SSTREAM
+     istringstream istr(signature.c_str());
+     ostringstream ostr;
+#else
      istrstream istr(signature.c_str());
      ostrstream ostr;
+#endif
+
      GPGInterface verifyer(G_App->getCfg().gpg.secretkeyid, _passphrase, _armor);
 
      // Verify the signature
