@@ -365,8 +365,8 @@ void PrettyJID::on_presence(const Presence& p, const Presence::Type prev)
      }
      if (x)
      {
-	  string song_title = x->findElement_getCDATA("title");
-	  string state = x->findElement_getCDATA("state");
+	  string song_title = x->getChildCData("title");
+	  string state = x->getChildCData("state");
 	  if (state == "stopped")
 	  {
 	       song_title += _(" [stopped]");
