@@ -14,7 +14,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
  *  Gabber
- *  Copyright (C) 1999-2002 Dave Smith & Julian Missig
+ *  Copyright (C) 1999-2003 Dave Smith & Julian Missig
  */
 
 #include "GabberConfig.hh" // Needed for the ifdef.. hrmm
@@ -138,7 +138,6 @@ GabberWin::GabberWin()
 
      // Status Indicator(s)
      StatusIndicator::display_presence(Presence("", Presence::ptUnavailable, Presence::stOffline, toUTF8(_thisWindow, _("Disconnected"))));
-     _dockWindow = StatusIndicator::get_dock_window();
 
      // Load spooled messages
      G_App->getMessageManager().load_spool();
@@ -373,10 +372,6 @@ void GabberWin::on_session_connected(const Element& t)
      // Play connected sound
      gnome_triggers_do(NULL, NULL, "gabber", "Connected", NULL);
 
-     // If they didn't want to save the password, clear it
-     if (!G_App->getCfg().server.savepassword)
-	  G_App->getCfg().server.password = "";
-
      // Disable login menuitem
      _m_Login->set_sensitive(false);
      _md_Login->set_sensitive(false);
@@ -415,12 +410,6 @@ void GabberWin::on_session_connected(const Element& t)
      if (WelcomeDruid::isRunning())
 	 	WelcomeDruid::Connected();
 
-     // Send out autoupdate request
-     string autoupdateJID = "956878967";             // Gabber's clientID on jabbercentral
-     autoupdateJID += "@update.jabber.org/";         // the only place to grab updates right now
-     autoupdateJID += ConfigManager::get_VERSION();  // Gabber's version
-     G_App->getSession() << Presence(autoupdateJID, Presence::ptAvailable);
-
      // Grab the Show, Status, and Priority for presence
      _curShow = indexShow(G_App->getCfg().get_show());
      _curStatus = G_App->getCfg().get_status();
@@ -569,35 +558,38 @@ void GabberWin::on_Manual_activate()
 
 void GabberWin::on_Show_selected(int custom_index)
 {
-     string status = _("Online");
+     string status = G_App->getCfg().get_status(); // config text is all UTF-8
 
      if (custom_index >= 0) // All normal show types
      {
-	  switch (indexShow(custom_index))
-	  {
-	  case Presence::stOffline:
-	       status = _("Done for the day. I'll get back to you tomorrow.");
-	       break;
-	  case Presence::stOnline:
-	       status = _("Online");
-	       break;
-	  case Presence::stChat:
-	       status = _("Chat with me!");
-	       break;
-	  case Presence::stAway:
-	       status = _("I'm away.");
-	       break;
-	  case Presence::stXA:
-	       status = _("I'll be away for a while.");
-	       break;
-	  case Presence::stDND:
-	       status = _("Sorry, I'm busy right now.");
-	       break;
-	  default:
-	       display_status(G_App->getCfg().get_show(), 
-			      G_App->getCfg().get_status(),
-			      G_App->getCfg().get_priority());
-	       return;
+	  // If we have no status message or the user selects another presence
+	  if (!status.length() || custom_index != _curShow) {
+		  switch (indexShow(custom_index))
+		  {
+		  case Presence::stOffline:
+		       status = toUTF8(_thisWindow, _("Done for the day. I'll get back to you tomorrow."));
+		       break;
+		  case Presence::stOnline:
+		       status = toUTF8(_thisWindow, _("Online"));
+		       break;
+		  case Presence::stChat:
+		       status = toUTF8(_thisWindow, _("Chat with me!"));
+		       break;
+		  case Presence::stAway:
+		       status = toUTF8(_thisWindow, _("I'm away."));
+		       break;
+		  case Presence::stXA:
+		       status = toUTF8(_thisWindow, _("I'll be away for a while."));
+		       break;
+		  case Presence::stDND:
+		       status = toUTF8(_thisWindow, _("Sorry, I'm busy right now."));
+		       break;
+		  default:
+		       display_status(G_App->getCfg().get_show(), 
+				      G_App->getCfg().get_status(),
+				      G_App->getCfg().get_priority());
+		       return;
+		  }
 	  }
 	  manage(new ShowDlg(indexShow(custom_index), status, _curPriority));
      }
@@ -818,20 +810,39 @@ void GabberWin::display_status(Presence::Show show, const string& current_status
 	  }
 #endif // HAVE_XMMS
 	  StatusIndicator::display_presence(p);
-	  _dockWindow = StatusIndicator::get_dock_window();
 	  // If the presence should be sent
 	  if (send_presence)
 	  {
-//FIXME: Need to add error-checking for the type="invisible" case!
 	       G_App->getSession() << p;
 	       // Fire off the event for my presence
 	       evtMyPresence(p);
+
+	       // If this is an invisible presence
+	       if (p.getType() == Presence::ptInvisible)
+	       {
+		    // In order for transports to know to show us as invisible,
+		    // we must send this presence directly to each of them
+		    for (Roster::iterator it = G_App->getSession().roster().begin(); 
+			 it != G_App->getSession().roster().end(); 
+			 ++it)
+		    {
+			 // If this is an agent
+			 if (G_App->isAgent(it->getJID()))
+			 {
+			      // Copy the presence
+			      Presence ptrans(p.getBaseElement());
+			      // Set it to be to the transport
+			      ptrans.setTo(it->getJID());
+			      // Send it off
+			      G_App->getSession() << ptrans;
+			 }
+		    }
+	       }
 	  }
      }
      else // We're not connected
      {
 	  StatusIndicator::display_presence(Presence("", Presence::ptUnavailable, Presence::stOffline, toUTF8(_thisWindow, _("Disconnected"))));
-	  _dockWindow = StatusIndicator::get_dock_window();
      }
      g_free(cpriority);
 
@@ -984,6 +995,12 @@ void GabberWin::toggle_visibility()
      }
 }
 
+void GabberWin::reconnecting()
+{
+     // Update the status bar   
+     push_status_bar_msg(_("Reconnecting..."), 0);
+}
+
 void GabberWin::logging_in()
 {
      // While it's trying to login, do this stuff:
