@@ -28,12 +28,14 @@
 #include "GabberWin.hh"
 #include "GCInterface.hh"
 #include "MessageViews.hh"
+#include "EventManager.hh"
 
+#include <fcntl.h>
 #include <string>
+#include <stdio.h>
 #include <dirent.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <libgnome/gnome-triggers.h>
 #include <libgnome/gnome-util.h>
 
 using namespace jabberoo;
@@ -136,22 +138,31 @@ void MessageManager::add(const jabberoo::Message& m, bool spool)
      else if (cfgm.msgs.recvooochats && (mtype == translateType("normal")))
           mtype = translateType("chat");
 
+     //XXX: This code ripped straight from GabberWidgets ... not good :(
+     //XXX: it's used for message_received below
+     // Lookup nickname - default is the username
+     string jid = m.getFrom();
+     string resource = JID::getResource(jid);
+     string nickname = JID::getUser(jid);
+     try {
+	  nickname = G_App->getSession().roster()[JID::getUserHost(jid)].getNickname();
+     } catch (Roster::XCP_InvalidJID& e) {
+	  // Special handling for a groupchat ID -- use the resource as the nickname
+	  if (G_App->isGroupChatID(jid) && !resource.empty())
+	       nickname = resource;
+     }
+
      // Select a view based on the message type
      ViewMap& vm = _mviews[mtype];
 
      // Attempt to locate a view for this JID
      ViewMap::iterator it = vm.find(user);
-     // If a view is found....
-     if (it != vm.end())
+     // If a view is found... and it doesn't want messages to be queued
+     if (it != vm.end() && !(_message_view_info[mtype]->flags & MessageViewInfo::vfQueueMessage))
      {
-	  // XXX: this needs to he changed in some way so that new event sounds
-	  // can be added as easily as new views.  
-	  if (m.getType() == translateType("normal"))
-	       // Play RecvMessage sound
-	       gnome_triggers_do(NULL, NULL, "gabber", "RecvMessage", NULL);
-	  else if (m.getType() == translateType("chat"))
-	       // Play RecvOOOChat sound
-	       gnome_triggers_do(NULL, NULL, "gabber", "RecvOOOChat", NULL);
+	  // Call the message received event
+	  G_App->getEventManager().message_received(m, nickname, mtype, false);
+
 	  // Pass the message on to the view
 	  it->second->render(m);
      }
@@ -166,6 +177,20 @@ void MessageManager::add(const jabberoo::Message& m, bool spool)
 	  else if (_message_view_info[mtype]->flags & MessageViewInfo::vfPopup)
 	       popup = true;
 
+	  // Or a view was found and we just want this queued
+	  if (it != vm.end()
+	      // Wanting this queued is implied from the other if statement which checks
+	      // for it != vm.end()
+	      // && (_message_view_info[mtype]->flags & MessageViewInfo::vfQueueMessage)
+	       )
+	  {
+	       // So we notify the view that we have messages waiting now
+	       it->second->has_messages_waiting(true);
+
+	       // And we do not want this to automatically popup
+	       popup = false;
+	  }
+
 	  // Lookup the message list for this JID and Message Type
 	  MList& mlist = _mlistmap[mtype][user];
 	  // Add the message
@@ -178,69 +203,67 @@ void MessageManager::add(const jabberoo::Message& m, bool spool)
 	       MSpoolMap::iterator it = _mspools[mtype].find(user);
 
 	       if (it == _mspools[mtype].end())
-               {
-		    string file = user + "-" + translateType(mtype);
-		    if (file.find("/") != string::npos)
-			 file.replace(file.find("/"), 1, "-");
-                    char *filename = g_concat_dir_and_file(_spool.c_str(), file.c_str());
-                    ofstream* spool = new ofstream(filename, ios::out | ios::ate);
-                    pair<MSpoolMap::iterator, bool> np = _mspools[mtype].insert(make_pair(user, spool));
-		    it = np.first;
+           {
+                string file = user + "-" + translateType(mtype);
+                if (file.find("/") != string::npos)
+                    file.replace(file.find("/"), 1, "-");
+                char *filename = g_concat_dir_and_file(_spool.c_str(), file.c_str());
+		if (!g_file_exists(filename))
+		{
+		     open(filename, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);
+		}
+                fstream* spool = new fstream(filename, ios::out | ios::ate);
+                pair<MSpoolMap::iterator, bool> np = _mspools[mtype].insert(make_pair(user, spool));
+                it = np.first;
 
-                    spool->seekp(0, ios::end);
-                    // If file is empty output proper XML header
-                    if (spool->tellp() == (streampos) 0)
-                    {
-                         *spool << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << endl;
-                         *spool << "<spool xmlns=\"http://gabber.sourceforge.net/namespace/spool\">" << endl;
-
-                         pos = spool->tellp();
-                         *spool << XML_FOOTER;
-                         spool->seekp(pos);
-                    }
-                    else
-                         spool->seekp(spool->tellp() - (streampos) XML_FOOTER.size());
-               }
+                spool->seekp(0, ios::end);
+                // If file is empty output proper XML header
+                if (spool->tellp() == (streampos) 0)
+                {
+                     *spool << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << endl;
+                     *spool << "<spool xmlns=\"http://gabber.sourceforge.net/namespace/spool\">" << endl;
+                     *spool << XML_FOOTER;
+                }
+           }
 	       Message logm(m);
 
 	       // If the message has no timestamp add one so the correct time is displayed
 	       Element* x = logm.findX("jabber:x:delay");
 	       if (!x)
 	       {
-	            time_t curtime = time(0);
-	            struct tm* t = gmtime(&curtime);
-	            t->tm_year += 1900;
-	            t->tm_mon += 1;
-	            char* time = g_strdup_printf("%04d%02d%02dT%02d:%02d:%02d", t->tm_year, t->tm_mon, t->tm_mday,
+               time_t curtime = time(0);
+               struct tm* t = gmtime(&curtime);
+               t->tm_year += 1900;
+               t->tm_mon += 1;
+               char* time = g_strdup_printf("%04d%02d%02dT%02d:%02d:%02d", t->tm_year, t->tm_mon, t->tm_mday,
 		 	       		         t->tm_hour, t->tm_min, t->tm_sec);
 
-		    Element* rx = logm.getBaseElement().addElement("x");
-		    rx->putAttrib("xmlns", "jabber:x:delay");
-		    rx->putAttrib("stamp", time);
+               Element* rx = logm.getBaseElement().addElement("x");
+               rx->putAttrib("xmlns", "jabber:x:delay");
+               rx->putAttrib("stamp", time);
 
-		    g_free(time);
+               g_free(time);
 	       }
-
-	       *it->second << logm.toString();
-
-               pos = it->second->tellp();
-               *it->second << XML_FOOTER;
-               it->second->seekp(pos);
-
-               it->second->flush();
+            
+           it->second->seekp( - (int)XML_FOOTER.size(), ios::end);
+	       *it->second << logm.toString() << endl;
+           *it->second << XML_FOOTER;
+           it->second->flush();
 	  }
 
 	  if (popup)
 	  {
+	       // Call the message received event
+	       G_App->getEventManager().message_received(m, nickname, mtype, false);
+
+	       // Display the message
 	       display(m.getFrom(), mtype);
 	  }
 	  else
 	  {
-	       // XXX: redo this too
-	       if (mtype == translateType("normal"))
-		    gnome_triggers_do(NULL, NULL, "gabber", "QueueMessage", NULL);
-	       else if (mtype == translateType("chat"))
-		    gnome_triggers_do(NULL, NULL, "gabber", "QueueOOOChat", NULL);
+	       // Call the message received event
+	       G_App->getEventManager().message_received(m, nickname, mtype, true);
+
 	       if (!(_message_view_info[mtype]->flags & MessageViewInfo::vfNoFlash))
 	       {
             cout << "adding event " << mtype << " for " << user << endl;
@@ -326,7 +349,7 @@ MessageView* MessageManager::display(const string jid, const MessageType type)
 			 // Attempt to locate an existing view..
 			 ViewMap::iterator it = vm.find(user);
 			 
-			 // If an existing view was found, use it to render the message...
+			 // If an existing view was found, tell it there are more messages where that came from
 			 if (it != vm.end())
 			 {
 			      view = it->second;
@@ -342,6 +365,8 @@ MessageView* MessageManager::display(const string jid, const MessageType type)
      // make sure a View of the requested type is displayed. 
      if (!view_displayed)
           view = render(jid, type);
+     if (view)
+	     view->raise();
      return view;
 }
 
@@ -507,6 +532,8 @@ void MessageManager::popEvent(const string& jid, MessageType type)
                _events.erase(it);
 	       break;
 	  }
+	  else
+	       ++it;
      }
      // If the user has no more events, clear the events on the roster
      if (getEvent(user) == _events.end())
@@ -647,7 +674,7 @@ MessageManager::MessageType MessageManager::translateType(const Message& m)
 {
      bool extension;
      string type;
-     Element *x = m.getBaseElement().findElement("x");
+     const Element *x = m.getBaseElement().findElement("x");
      MessageType mtype = 0;
 
      if (x)
