@@ -26,6 +26,7 @@
 #include "ContactInfoInterface.hh"
 #include "ContactInterface.hh"
 #include "ErrorManager.hh"
+#include "EventManager.hh"
 #include "GabberApp.hh"
 #include "GabberGPG.hh"
 #include "GabberUtility.hh"
@@ -105,6 +106,9 @@ void MessageView::has_messages_waiting(bool msgswaiting)
 {
 }
 
+void MessageView::raise() {
+}
+
 
 // -------------------------------------------------------------------
 //
@@ -268,7 +272,6 @@ void MessageSendDlg::init()
      _pjid = manage(new PrettyJID(_jid, "", PrettyJID::dtNickRes, 128, true, _jid.empty()));
      _pjid ->changed.connect(slot(this, &MessageSendDlg::on_SendTo_changed));
      _pjid ->show();
-     on_SendTo_changed(); // Call this to get the window title set
      getWidget<Gtk::HBox>("MessageSend_JIDInfo_hbox")->pack_end(*_pjid, true, true, 0);
      _nickname = _pjid->get_nickname();
      _onRoster = _pjid->is_on_roster();
@@ -285,6 +288,8 @@ void MessageSendDlg::init()
      tb->set_sensitive(G_App->getGPG().enabled());
      tb->set_active(false);
 
+     on_SendTo_changed(); // Call this to get the window title set
+
      _thisWindow->set_default_size(G_App->getCfg().msgs.width, G_App->getCfg().msgs.height);
      _thisWindow->show();
 }
@@ -508,7 +513,11 @@ MessageView* MessageRecvView::new_view_jid(const string& jid, ViewMap& vm)
 }
 
 MessageRecvView::MessageRecvView(const jabberoo::Message& m, ViewMap& vm)
-     : MessageView(m, vm), BaseGabberWindow("MessageRecv_win")
+     : MessageView(m, vm), BaseGabberWindow("MessageRecv_win"),
+       _message(m),
+       _Messages_current(_Messages.begin()),
+       _msgswaiting(false),
+       _Messages_should_queue(true)
 {
      init();
 
@@ -527,6 +536,9 @@ MessageRecvView::~MessageRecvView()
 	  delete _composing_msg;
 	  _composing_msg = NULL;
      }
+     
+     // Clear out the queue
+     _Messages.clear();
 }
 
 void MessageRecvView::init()
@@ -547,6 +559,10 @@ void MessageRecvView::init()
      // Toolbar
      getWidget<Gtk::Toolbar>("MessageRecv_toolbar")->set_style(GTK_TOOLBAR_ICONS);
 
+     // Initialize pointer to "prev" button (should be insensitive by default)
+     _btnReadPrev = getButton("MessageRecv_ReadPrev_btn");
+     _btnReadPrev->clicked.connect(slot(this, &MessageRecvView::on_ReadPrev_clicked));
+     _btnReadPrev->set_sensitive(false);
      // Initialize pointer to "next" button (should be insensitive by default)
      _btnReadNext = getButton("MessageRecv_ReadNext_btn");
      _btnReadNext->clicked.connect(slot(this, &MessageRecvView::on_ReadNext_clicked));
@@ -566,6 +582,7 @@ void MessageRecvView::init()
      pj->show();
      getWidget<Gtk::HBox>("MessageRecv_JIDInfo_hbox")->pack_start(*pj, true, true, 0);
      _onRoster = pj->is_on_roster();
+     _nickname = pj->get_nickname();
      
      // Update the add contact button
      if (pj->is_on_roster())
@@ -586,9 +603,6 @@ void MessageRecvView::init()
      gnome_window_icon_set_from_file(_thisWindow->gtkobj(),window_icon.c_str());
      gnome_window_icon_init();
 
-     // _useGPG will be set when a message is rendered
-     _useGPG = MessageUtil::gpg_toggle(getWidget<Gtk::ToggleButton>("MessageRecv_Encrypt_tglbtn"), JID::getUserHost(_jid), JID::getResource(_jid));
-
      // Add button_press handler for GPGInfo Dialog
      _evtGPG->button_press_event.connect(slot(this, &MessageRecvView::on_GPGInfo_button_press));
 
@@ -598,6 +612,33 @@ void MessageRecvView::init()
 
 void MessageRecvView::render(const jabberoo::Message& m)
 {
+     // if we should be adding it to the queue
+     if (_Messages_should_queue)
+     {
+	  // Add the message to the queue
+	  _Messages.push_back(m);
+	  _Messages_current = --_Messages.end();
+     }
+
+     // This is the last message in the queue
+     if (_Messages_current == --_Messages.end())
+     {	  
+	  // Set Read Next sensitivity -- in case we just came to the font
+	  _btnReadNext->set_sensitive(_msgswaiting);
+     }
+     // we're somewhere within the queue
+     else
+	  _btnReadNext->set_sensitive(true);
+
+     _Messages_should_queue = true;
+
+     // If this is the first message we put in the queue 
+     // then we have no previous messages
+     if (_Messages_current == _Messages.begin())
+	  _btnReadPrev->set_sensitive(false);
+     else
+	  _btnReadPrev->set_sensitive(true);
+
      // Clear out the body text
      _txtBody->delete_text(0, -1);
      // Remove any extras added to the header table
@@ -716,9 +757,6 @@ void MessageRecvView::render(const jabberoo::Message& m)
 	  // Set the tooltip
 	  _tips.set_tip(*_evtGPG, _("This message is encrypted"));
 	  _gpgInfo = GabberGPG::gpgValidEncrypted;
-	  // This message was encrypted for enable encryption for outgoing messages
-//	       getWidget<Gtk::ToggleButton>("MessageRecv_Encrypt_tglbtn")->set_active(true);
-//	       _useGPG = true;
      }
 
      // Walk the list of children to find all extensions which can occur multiple times
@@ -755,8 +793,8 @@ void MessageRecvView::render(const jabberoo::Message& m)
 	       lbl->show();
 		    
 	       // The href
-	       string desc = x.findElement_getCDATA("desc");
-	       string url = x.findElement_getCDATA("url");
+	       string desc = x.getChildCData("desc");
+	       string url = x.getChildCData("url");
 	       // Ensure the desc is not too long
 	       if (desc.length() > 128)
 	       {
@@ -792,16 +830,15 @@ void MessageRecvView::render(const jabberoo::Message& m)
 	  }
      }
 
-     // If there appear to be events in this message
-     x = m.findX("jabber:x:event");
-     if (x != NULL)
+     // current message
+     if (x != NULL && _Messages_current == --_Messages.end())
      {
-	  if (x->findElement("displayed") != NULL)
-	  {
-	       // displayed event was requested and this message has been displayed
-	       G_App->getSession() << m.displayed();
-	  }
-	  if (x->findElement("composing") != NULL)
+	  // message displayed event
+	  G_App->getEventManager().message_displayed(m, _nickname, MessageManager::translateType(m));
+
+	  // If there appear to be events in this message
+	  x = m.findX("jabber:x:event");
+	  if (x != NULL && x->findElement("composing") != NULL)
 	  {
 	       if (_composing_msg != NULL)
 		    delete _composing_msg;
@@ -817,8 +854,12 @@ void MessageRecvView::render(const jabberoo::Message& m)
 
 void MessageRecvView::has_messages_waiting(bool msgswaiting)
 {
-     // Enable the next button, if necessary
-     _btnReadNext->set_sensitive(msgswaiting);
+     _msgswaiting = msgswaiting;
+     if (_Messages_current == --_Messages.end())
+     {
+	  // Enable the next button, if necessary
+	  _btnReadNext->set_sensitive(msgswaiting);
+     }
 }
 
 void MessageRecvView::on_Reply_clicked()
@@ -832,17 +873,33 @@ void MessageRecvView::on_Reply_clicked()
      }
 
      // Open
-     manage(new MessageSendDlg(_jid));
+     manage(new MessageSendDlg(_message, this));
+}
+
+void MessageRecvView::on_ReadPrev_clicked()
+{
+     _Messages_should_queue = false;
+     render(*(--_Messages_current));
 }
 
 void MessageRecvView::on_ReadNext_clicked()
 {
-     MessageManager::EventList::iterator it = G_App->getMessageManager().getEvent(_jid);
-     if (it != G_App->getMessageManager().getEvents().end())
+     if (_Messages_current == --_Messages.end())
      {
-	  // display using it->first instead of defaultJID because defaultJID may not
-	  // contain a resource or may not be the resource the event came from
-	  G_App->getMessageManager().display(it->first, it->second);
+	  // We want to go grab a new message
+	  MessageManager::EventList::iterator it = G_App->getMessageManager().getEvent(_jid);
+	  if (it != G_App->getMessageManager().getEvents().end())
+	  {
+	       // display using it->first instead of defaultJID because defaultJID may not
+	       // contain a resource or may not be the resource the event came from
+	       G_App->getMessageManager().display(it->first, it->second);
+	  }
+     }
+     else
+     {
+	  _Messages_should_queue = false;
+	  // We're somewhere within the queue
+	  render(*(++_Messages_current));
      }
 }
 
@@ -1022,7 +1079,7 @@ void ChatMessageView::init()
      _onRoster = pj->is_on_roster();
 
 
-     _baseCloseChat->getLabel("OOOChat_CloseChat_lbl")->set_text(substitute(_("You just received a message from %s."), pj->get_nickname())); 
+     _baseCloseChat->getLabel("OOOChat_CloseChat_lbl")->set_text(substitute(_("You just received a message from %s."), fromUTF8(_thisWindow, pj->get_nickname()))); 
 
      // Create chat view
      _chatview = new ChatView(_thisWindow, _hboxChatview, false);
@@ -1091,6 +1148,15 @@ void ChatMessageView::init()
      _thisWindow->show();
 }
 
+void ChatMessageView::raise() {
+     // If this window is visible
+     if (_thisWindow->is_visible()) {
+	  // Raise the window
+	  gdk_window_show(_thisWindow->get_window().gdkobj());
+	  _thisWindow->activate_focus();
+     }
+}
+
 void ChatMessageView::get_status()
 {
      try {
@@ -1123,8 +1189,8 @@ void ChatMessageView::render(const jabberoo::Message& m)
      // Grab the subject and thread, since they may have changed
      _subject = m.getSubject();
      _thread = m.getThread();
-
      string message = m.getBody();
+
      // reset useGPG so signed messages are only sent if the received message was signed or encrypted
      _useGPG = false;
      // Check for a signature
@@ -1214,25 +1280,23 @@ void ChatMessageView::render(const jabberoo::Message& m)
 			    fromUTF8(GTK_WIDGET(_chatview->_xtext), _nickname),
                             "", RED);
 
+     // message displayed event
+     G_App->getEventManager().message_displayed(m, _nickname, MessageManager::translateType(m));
+     
      // If there appear to be events in this message
      x = m.findX("jabber:x:event");
-     if (x != NULL)
+     
+     if (x != NULL && x->findElement("composing") != NULL)
      {
-	  if (x->findElement("displayed") != NULL)
-	  {
-	       // displayed event was requested and this message has been displayed
-	       G_App->getSession() << m.displayed();
-	  }
-	  if (x->findElement("composing") != NULL)
-	  {
-	       if (_composing_msg != NULL)
-		    delete _composing_msg;
-	       // Get the composing event message
-	       _composing_msg = new Message(m.composing().getBaseElement());
-
-	       // hook up a connection to send the composing event
-	       _composing_event = _memMsg->changed.connect(slot(this, &ChatMessageView::on_memMsg_changed));
-	  }
+	  if (_composing_msg != NULL)
+	       delete _composing_msg;
+	  // Get the composing event message
+	  _composing_msg = new Message(m.composing().getBaseElement());
+	  
+	  if (_composing_event.connected())
+	       _composing_event.disconnect();
+	  // hook up a connection to send the composing event
+	  _composing_event = _memMsg->changed.connect(slot(this, &ChatMessageView::on_memMsg_changed));
      }
 	       
      // Popup the window if they like it that way
@@ -1318,6 +1382,12 @@ void ChatMessageView::on_Send_clicked()
 		    _tips.set_tip(*_evtGPG, _("This message is encrypted"));
 	       }
           }
+
+	  if (_composing_msg != NULL && !_composing_event.connected())
+	  {
+	       // hook up a connection to send the composing event
+	       _composing_event = _memMsg->changed.connect(slot(this, &ChatMessageView::on_memMsg_changed));
+	  }
      }
 }
 
@@ -1421,13 +1491,11 @@ gint ChatMessageView::on_window_key_press(GdkEventKey* e)
 void ChatMessageView::on_memMsg_changed()
 {
      // composing event was requested and a reply is being typed
-     if (_composing_msg != NULL)
+     if (_composing_msg != NULL && _memMsg->get_length() > 0)
      {
 	  G_App->getSession() << *_composing_msg;
-	  delete _composing_msg;
-	  _composing_msg = NULL;
+	  _composing_event.disconnect();
      }
-     _composing_event.disconnect();
 }
 
 void ChatMessageView::on_AddUser_clicked()
@@ -1597,7 +1665,7 @@ GCMessageView::~GCMessageView()
     for (; it != _lstUsers->rows().end(); it++)
     {
        Gtk::CList::Row& row = *(it);
-       std::string nick = row[0].get_text();
+       std::string nick = toUTF8(_lstUsers, row[0].get_text());
        
        if(nick != _nick)
        {
@@ -1706,7 +1774,7 @@ void GCMessageView::init()
 	  _lstUsers->set_row_height(17);                                        // Hack for making sure icons aren't chopped off
 
      // Groupchat nicknames are case sensitive, so we use a case sensitive compare
-     _lstUsers->set_compare_func(&GabberUtil::strcmp_clist_items);
+     _lstUsers->set_compare_func(&GabberUtil::strcasecmp_clist_items);
 
      // Setup tab-completion buffer
      _completer = g_completion_new(NULL);
@@ -1762,11 +1830,25 @@ void GCMessageView::render(const jabberoo::Message& m)
      // A message from someone else
      else
      {
+	  bool highlight = false;
+	  if (body.find(_nick) != string::npos)
+	       highlight = true;
+
           // If they want timestamps, put them in
           if (G_App->getCfg().chats.grouptime)
-               _chatview->render(body, fromnick, m.getDateTime(G_App->getCfg().dates.gcformat), RED);
+	  {
+	       if (highlight)
+		    _chatview->render_highlight(body, fromnick, m.getDateTime(G_App->getCfg().dates.gcformat), RED, RED);
+	       else
+		    _chatview->render(body, fromnick, m.getDateTime(G_App->getCfg().dates.gcformat), RED);
+	  }
           else
-               _chatview->render(body, fromnick, "", RED);
+	  {
+	       if (highlight)
+		    _chatview->render_highlight(body, fromnick, "", RED, RED);
+	       else
+		    _chatview->render(body, fromnick, "", RED);
+	  }
      }
 
      const string& subj = m.getSubject();
@@ -1776,6 +1858,12 @@ void GCMessageView::render(const jabberoo::Message& m)
 	  //_thisWindow->set_title(_("Gabber: Group Chat - ") + fromUTF8(_thisWindow, subj));
 	  _entSubject->set_text(fromUTF8(_entSubject, subj));
      }
+
+     // message displayed event
+     if (fromnick.empty())
+	  fromnick = _("Server");
+     if (fromnick != _nick) // If it's not from us
+	  G_App->getEventManager().message_displayed(m, fromnick, MessageManager::translateType(m));
 }
 
 gint GCMessageView::on_window_delete(GdkEventAny* e)
@@ -1796,7 +1884,7 @@ gint GCMessageView::on_Subject_key_press(GdkEventKey* e)
           if (!subject.empty())
           {
                // Setup a message object
-               Message m(_room, substitute(_("%s has changed the subject to: %s"), "/me", subject), Message::mtGroupchat);
+               Message m(_room, substitute(toUTF8(_entSubject, _("%s has changed the subject to: %s")), "/me", subject), Message::mtGroupchat);
                m.setSubject(subject);
                // Transmit the subject change
                G_App->getSession() << m;
@@ -1815,21 +1903,24 @@ int GCMessageView::on_Roster_button_press(GdkEventButton* e)
      switch (e->type) {
      case GDK_2BUTTON_PRESS: // Double-click
      {
-          // Check for control key..
-          GdkEventButton* ex = (GdkEventButton*)e;
-          if (ex->state & GDK_CONTROL_MASK)
-	  {
-	       if (G_App->getCfg().msgs.sendmsgs)
-                    G_App->getMessageManager().display(_room + "/" + toUTF8(_lstUsers, _lstUsers->cell(_row, _col).get_text()), MessageManager::translateType("chat"));
+	  if (e->button == 1)
+	  { 
+	       // Check for control key..
+	       GdkEventButton* ex = (GdkEventButton*)e;
+	       if (ex->state & GDK_CONTROL_MASK)
+	       {
+		    if (G_App->getCfg().msgs.sendmsgs)
+			 G_App->getMessageManager().display(_room + "/" + toUTF8(_lstUsers, _lstUsers->cell(_row, _col).get_text()), MessageManager::translateType("chat"));
+		    else
+			 G_App->getMessageManager().display(_room + "/" + toUTF8(_lstUsers, _lstUsers->cell(_row, _col).get_text()), MessageManager::translateType("normal"));
+	       }
 	       else
-		    G_App->getMessageManager().display(_room + "/" + toUTF8(_lstUsers, _lstUsers->cell(_row, _col).get_text()), MessageManager::translateType("normal"));
-	  }
-          else
-	  {
-	       if (G_App->getCfg().msgs.sendmsgs)
-		    G_App->getMessageManager().display(_room + "/" + toUTF8(_lstUsers, _lstUsers->cell(_row, _col).get_text()), MessageManager::translateType("normal"));
-	       else
-		    G_App->getMessageManager().display(_room + "/" + toUTF8(_lstUsers, _lstUsers->cell(_row, _col).get_text()), MessageManager::translateType("chat"));
+	       {
+		    if (G_App->getCfg().msgs.sendmsgs)
+			 G_App->getMessageManager().display(_room + "/" + toUTF8(_lstUsers, _lstUsers->cell(_row, _col).get_text()), MessageManager::translateType("normal"));
+		    else
+			 G_App->getMessageManager().display(_room + "/" + toUTF8(_lstUsers, _lstUsers->cell(_row, _col).get_text()), MessageManager::translateType("chat"));
+	       }
 	  }
 	  break;
      }
@@ -1892,13 +1983,14 @@ void GCMessageView::on_Users_toggled()
      {
 	  getWidget<Gtk::VBox>("GC_UserList_vbox")->show();
 	  _hpane->set_gutter_size(12);
-          _hpane->set_position(_posHpane);
+	  _hpane->set_position(_thisWindow->width() - _posHpane);
      }
      else
      {
           // if they don't, remember old position, and set to a big number. xchat uses 1200
+          _posHpane = _thisWindow->width() - (_hpane->gtkobj()->handle_xpos + 12); // Add the gutter size. Yeah, that kinda sucks
+	  // At this point, _posHane holds the width of the GroupChat contact list, not the position.
 	  _hpane->set_position(2400);
-          _posHpane = _hpane->gtkobj()->handle_xpos - 12; // Subtract the gutter size. Yeah, that kinda sucks
 	  _hpane->set_gutter_size(0);
 	  getWidget<Gtk::VBox>("GC_UserList_vbox")->hide();
      }
@@ -2135,7 +2227,7 @@ bool GCMessageView::process_command(const string& cmd)
      // NICK handler
      if ((cmd.substr(0,6) == "/nick ") && (cmd.length() > 6))
      {
-          string newnick = toUTF8(_memMessage, cmd.substr(6));
+          string newnick = cmd.substr(6);
           if (!newnick.empty())
           {
 	       _oldnick = _nick;
@@ -2147,7 +2239,7 @@ bool GCMessageView::process_command(const string& cmd)
      // QUERY handler
      else if ((cmd.substr(0,6) == "/query") && (cmd.length() > 7))
      {
-          string user = toUTF8(_memMessage, cmd.substr(7));
+          string user = cmd.substr(7);
           if (!user.empty())
                G_App->getMessageManager().display(_room + "/" + user, MessageManager::translateType("chat"));
           return true;
@@ -2155,7 +2247,7 @@ bool GCMessageView::process_command(const string& cmd)
      // MSG handler
      else if ((cmd.substr(0,4) == "/msg") && (cmd.length() > 5))
      {
-          string user = toUTF8(_memMessage, cmd.substr(5));
+          string user = cmd.substr(5);
           if (!user.empty())
                G_App->getMessageManager().display(_room + "/" + user, MessageManager::translateType("normal"));
           return true;
@@ -2163,9 +2255,9 @@ bool GCMessageView::process_command(const string& cmd)
      // TOPIC handler
      else if ((cmd.substr(0,6) == "/topic") && (cmd.length() > 7))
      {
-	  string subject = toUTF8(_memMessage, cmd.substr(7));
+	  string subject = cmd.substr(7);
 	  // Setup a message object
-	  Message m(_room, _("/me has changed the subject to: ") + subject, Message::mtGroupchat);
+	  Message m(_room, substitute(toUTF8(_entSubject, _("%s has changed the subject to: %s")), "/me", subject), Message::mtGroupchat);
 	  m.setSubject(subject);
 	  // Transmit the subject change
 	  G_App->getSession() << m;
@@ -2174,7 +2266,7 @@ bool GCMessageView::process_command(const string& cmd)
      // WHOIS handler
      else if ((cmd.substr(0,6) == "/whois") && (cmd.length() > 7))
      {
-	  string gcuser = toUTF8(_memMessage, cmd.substr(7));
+	  string gcuser = cmd.substr(7);
 	  ContactInfoDlg::display(_room + "/" + gcuser, Roster::rsNone, _thisWindow);
 	  return true;
      }
@@ -2202,7 +2294,7 @@ bool GCMessageView::process_command(const string& cmd)
      {
 	  string status;
 	  if (cmd.length() > 7)
-	       status = toUTF8(_memMessage, cmd.substr(7));
+	       status = cmd.substr(7);
 	  else
 	       status = _("I'm away.");
 	  G_Win->display_status(jabberoo::Presence::stAway, status, G_App->getCfg().get_priority());
@@ -2211,7 +2303,7 @@ bool GCMessageView::process_command(const string& cmd)
      // JOIN handler
      else if ((cmd.substr(0,5) == "/join") && (cmd.length() > 6))
      {
-	  string newroom = toUTF8(_memMessage, cmd.substr(6));
+	  string newroom = cmd.substr(6);
 	  if (newroom.find("@") == string::npos)
 	  {
 	       // Join given_room @ current_server / current_nick
@@ -2422,7 +2514,7 @@ void GCJoinDlg::changed()
 
 void GCJoinDlg::on_agents_reply(const Element& iq)
 {
-     Element* query = iq.findElement("query");
+     const Element* query = iq.findElement("query");
      Element::const_iterator it = query->begin();
 
      if (iq.cmpAttrib("type", "result")) 
@@ -2508,27 +2600,27 @@ void AutoupdateDlg::on_update_reply(const Element& t)
 {
      if (!t.empty())
      {
-          Element* query = t.findElement("query");
+          const Element* query = t.findElement("query");
           if (query != NULL)
           {
-               Element* release = query->findElement("release");
+               const Element* release = query->findElement("release");
                string curtext;
                if (release)
                {
                     // If they've already seen this update and don't want to see it again,
 		    // close the update window.  (It isn't displayed yet but close to 
 		    // destory it).  
-                    if (G_App->getCfg().server.lastupdate == release->findElement_getCDATA("version"))
+                    if (G_App->getCfg().server.lastupdate == release->getChildCData("version"))
                          close();
 
 		    // Fill in information about the update
 		    Gtk::Label* l = getLabel("Autoupdate_UpdateVer_lbl");
-		    l->set_text(fromUTF8(l, (string("Gabber ") + release->findElement_getCDATA("version"))));
+		    l->set_text(fromUTF8(l, (string("Gabber ") + release->getChildCData("version"))));
 		    l = getLabel("Autoupdate_UpdateDesc_lbl");
-		    l->set_text(fromUTF8(l, release->findElement_getCDATA("desc")));
+		    l->set_text(fromUTF8(l, release->getChildCData("desc")));
 		    Gnome::HRef* b = getWidget<Gnome::HRef>("Autoupdate_UpdateURI_href");
-		    b->set_url(fromUTF8(b, release->findElement_getCDATA("url")));
-		    b->set_label(fromUTF8(b, release->findElement_getCDATA("url")));
+		    b->set_url(fromUTF8(b, release->getChildCData("url")));
+		    b->set_label(fromUTF8(b, release->getChildCData("url")));
 
 		    // show the view
 		    show();
