@@ -263,7 +263,7 @@ void LoginDlg::saveconfig()
      c.status.priority = _spinPriority->get_value_as_int();
      c.server.resource = toUTF8(_entResource, _entResource->get_text());
      c.server.savepassword =_chkSavePassword->get_active();
-     // If they want to save the password, write it, it will be overwritten after login
+     // If they want to save the password, write it, it will not be saved by ConfigManager
      c.server.password = toUTF8(_entPassword, _entPassword->get_text());
      c.server.plaintext = _chkPlaintext->get_active();
      c.server.ssl = _chkSSL->get_active();
@@ -361,11 +361,16 @@ void LoginDlg::proxy_changed()
      {
      case ConfigManager::proxySOCKS5:
      case ConfigManager::proxyHTTP:
+	  getFrame("Login_ProxyServer_frame")->set_sensitive(true);
 	  getFrame("Login_ProxyAuth_frame")->set_sensitive(true);
 	  break;
-     case ConfigManager::proxyNone:
      case ConfigManager::proxySOCKS4:
+	  getFrame("Login_ProxyServer_frame")->set_sensitive(true);
+	  getFrame("Login_ProxyAuth_frame")->set_sensitive(false);
+	  break;
+     case ConfigManager::proxyNone:
      default:
+	  getFrame("Login_ProxyServer_frame")->set_sensitive(false);
 	  getFrame("Login_ProxyAuth_frame")->set_sensitive(false);
      }
 }
@@ -518,6 +523,8 @@ PrefsWin::PrefsWin()
      // Encryption
      _chkGPGEnable = getCheckButton("Prefs_GPG_Enable_chk");
      _chkGPGEnable ->toggled.connect(slot(this, &PrefsWin::changed));
+     _entGPGKeyserver = getEntry("Prefs_GPG_Keyserver_ent");
+//     _entGPGKeyserver ->changed.connect(slot(this, &PrefsWin::changed));
      _clistGPGKey  = getWidget<Gtk::CList>("Prefs_GPG_Key_clist");
      _clistGPGKey  ->select_row.connect(slot(this, &PrefsWin::on_GPGKey_select_row));
      // whether the secret keys have been retrieved from gpg and added to the clist yet
@@ -609,7 +616,8 @@ void PrefsWin::loadconfig()
      // enabled in the config file
      bool value = c.gpg.enabled;
      _chkGPGEnable->set_active(value);
-     cerr << "setting to " << value << endl;
+     _entGPGKeyserver->set_text(fromUTF8(c.gpg.keyserver));
+     G_App->getCfg().loadEntryHistory(getGEntry("Prefs_GPG_Keyserver_gent"));
      if (gpg.find_gpg() && !_gotKeys)
      {
 	  for (unsigned int i = 0; i < _clistGPGKey->columns().size(); i++)
@@ -626,7 +634,7 @@ void PrefsWin::loadconfig()
                string key = c.gpg.secretkeyid;
                for ( ; it != keys.end(); it++)
                {
-                    const char* rowdata[2] = { it->get_userid().c_str(), it->get_keyid().c_str() };
+                    const char* rowdata[2] = { fromUTF8(it->get_userid()).c_str(), it->get_keyid().c_str() };
 	            gint row = _clistGPGKey->append_row(rowdata);
 	            if (it->get_keyid() == key)
 	                 _clistGPGKey->row(row).select();
@@ -638,7 +646,10 @@ void PrefsWin::loadconfig()
 	  _clistGPGKey->thaw();
      }
      else if (!gpg.find_gpg())
+     {
 	  getWidget<Gtk::Frame>("Prefs_GPG_Key_frm")->set_sensitive(false);
+	  getWidget<Gtk::Table>("Prefs_GPG_extra_tbl")->set_sensitive(false);
+     }
 
      // Auto-away
      _chkAutoAway  ->set_active(c.autoaway.enabled);
@@ -718,6 +729,8 @@ void PrefsWin::saveconfig()
      if (c.gpg.enabled != _chkGPGEnable->get_active())
 	  gpg_changed = true;
      c.gpg.enabled = _chkGPGEnable->get_active();
+     c.gpg.keyserver = toUTF8(_entGPGKeyserver->get_text());
+     G_App->getCfg().saveEntryHistory(getGEntry("Prefs_GPG_Keyserver_gent"));
      // If the clist has a selection, store the keyid
      if (_clistGPGKey->selection().begin() != _clistGPGKey->selection().end())
      {
@@ -794,10 +807,10 @@ void PrefsWin::changed()
                string key = G_App->getCfg().gpg.secretkeyid;
                for ( ; it != keys.end(); it++)
                {
-                    const char* rowdata[2] = { it->get_userid().c_str(), it->get_keyid().c_str() };
+                    const char* rowdata[2] = { fromUTF8(it->get_userid()).c_str(), it->get_keyid().c_str() };
                     gint row = _clistGPGKey->append_row(rowdata);
                     if (it->get_keyid() == key)
-                    _clistGPGKey->row(row).select();
+			 _clistGPGKey->row(row).select();
                }
 	       _gotKeys = true;
 	  }
@@ -805,11 +818,14 @@ void PrefsWin::changed()
 	       _clistGPGKey->set_column_resizeable(i, true);
 	  _clistGPGKey->thaw();
 	  getWidget<Gtk::Frame>("Prefs_GPG_Key_frm")->set_sensitive(true);
+	  getWidget<Gtk::Table>("Prefs_GPG_extra_tbl")->set_sensitive(true);
+
      }
      // Otherwise diable the GPG frame and disbale the GPG check
      else
      {
 	  getWidget<Gtk::Frame>("Prefs_GPG_Key_frm")->set_sensitive(false);
+	  getWidget<Gtk::Table>("Prefs_GPG_extra_tbl")->set_sensitive(false);
 	  _chkGPGEnable->set_active(false);
      }
 
@@ -930,5 +946,4 @@ void PrefsWin::on_GPGKey_select_row(gint row, gint col, GdkEvent* e)
 {
      // mark the preferences box as changed when the users changes what key is selected
      // in the CList
-     
 }
