@@ -308,6 +308,17 @@ gint RosterView::on_refresh()
      float hscroll, vscroll;
      hscroll = _tscroll->get_hadjustment()->get_value(); // Save the horizontal scroll position
      vscroll = _tscroll->get_vadjustment()->get_value(); // Save the vertical scroll position
+
+     // Get the current selected user, if any
+     bool selectedUser = false;
+     string selectedJID;
+     if (!_tree->selection().empty())
+     {
+	  selectedUser = true;
+	  Gtk::CTree::Row r = *_tree->selection().begin();
+	  selectedJID = r[colJID].get_text();
+     }
+     
       _tree->clear();
 
      // Clear the groups nodemap
@@ -373,6 +384,21 @@ gint RosterView::on_refresh()
      _tscroll->get_hadjustment()->set_value(hscroll); // Set the horizontal scroll position
      _tscroll->get_vadjustment()->set_value(vscroll); // Set the horizontal scroll position
 
+
+     // Select the user which was selected before the refresh
+     if (selectedUser)
+     {
+	  for (Gtk::CTree_Helpers::RowIterator it = _tree->rows().begin(); it != _tree->rows().end(); it++)
+	  {
+	       if ((*it)[colJID].get_text() == selectedJID)
+	       {
+		    it->select();
+		    break;
+	       }
+	       // If we don't find the user, then do nothing...
+	  }
+     }
+     
      // Thaw the tree 
      _tree->thaw();
 
@@ -456,12 +482,11 @@ int RosterView::on_button_press(GdkEventButton* e)
      case GDK_2BUTTON_PRESS: // Double-click
      {
 	  // Only open message dialog for left button clicks
-	  if (e->button != 1)
-	       break;
-
-	  // Give the function the JabberID and whether control key is pressed
-	  on_roster_row_activate(_tree->get_text(_row, colJID), e->state & GDK_CONTROL_MASK);
-
+	  if (e->button == 1)
+	  {
+	       // Give the function the JabberID and whether control key is pressed
+	       on_roster_row_activate(_tree->get_text(_row, colJID), e->state & GDK_CONTROL_MASK);
+	  }
 	  break;
      }
      case GDK_BUTTON_PRESS: // Single-click
@@ -563,6 +588,13 @@ void RosterView::on_roster_row_activate(const string& jid, bool control_pressed)
 	  refresh();
 
 	  return;
+     } else {
+	it = G_App->getMessageManager().getEvent(jid);
+	if (it != G_App->getMessageManager().getEvents().end()){
+	  G_App->getMessageManager().display(it->first, it->second);
+	  refresh();
+	  return;
+	} 
      }
 
      // Check for control key..
@@ -582,6 +614,7 @@ void RosterView::on_roster_row_activate(const string& jid, bool control_pressed)
 	  else
 	       G_App->getMessageManager().display(defaultJID, MessageManager::translateType("chat"));
      }
+
 }
 
 // Right-click menu for users
@@ -656,12 +689,13 @@ void RosterView::on_S10nRequest_activate()
 void RosterView::on_DeleteUser_activate()
 {
      Gnome::Dialog* d;
-     string jid = toUTF8(_tree, _tree->get_text(_row, colJID));
-     string nickname = toUTF8(_tree, _tree->get_text(_row, colNickname));
+     Gtk::CTree::Row r = *_tree->selection().begin();
+     string jid = r[colJID].get_text();  // The jid is stored in the Gtk:: CTree in UTF-8,
+     string nickname = toUTF8(r[colNickname].get_text()); // while the nickname is in the local encoding.
 
      string question = substitute(_("Are you sure you want to remove %s from your roster?"), 
 				  fromUTF8(_tree, nickname) + " (" + fromUTF8(_tree, jid) + ")");
-     d = manage(Gnome::Dialogs::question_modal(question,
+     d = manage(Gnome::Dialogs::question_modal(question, // handle_RemoveUserDlg expects both string arguments to be in UTF-8.
 					       bind(slot(this, &RosterView::handle_RemoveUserDlg), jid, nickname)));
      main_dialog(d);
 }
@@ -704,6 +738,10 @@ void RosterView::handle_RemoveUserDlg(int code, string jid, string nickname)
      case 0:
 	  if (G_App->isAgent(jid))
 	  {
+	       // First send an unavailable presence so that
+	       // all users of that transport go offline
+	       G_App->getSession() << Presence(jid, Presence::ptUnavailable);
+
 	       // Send an iq request to get the key and thus unregister the agent
 	       id = G_App->getSession().getNextID();
 	       iq.setID(id);
@@ -757,10 +795,10 @@ void RosterView::handle_RemoveAgent_iq(const judo::Element& iq)
 {
      if (iq.cmpAttrib("type", "result"))
      {
-	  Element* query = iq.findElement("query");
+	  const Element* query = iq.findElement("query");
 	  if (query)
 	  {
-	       string key = query->findElement_getCDATA("key");
+	       string key = query->getChildCData("key");
 	       if (!key.empty())
 	       {
 		    // Unregister the agent
@@ -1177,7 +1215,7 @@ gboolean RosterView::is_drop_ok(GtkCTree* ctree, GtkCTreeNode* node, GtkCTreeNod
 
 void RosterView::dnd_get_vCard(const Element& iq)
 {
-     Element* vCard = iq.findElement("vCard");
+     const Element* vCard = iq.findElement("vCard");
 
      if (vCard)
      {
@@ -1717,7 +1755,7 @@ void SimpleRosterView::on_drag_data_received(GdkDragContext* drag_ctx,
 
 void SimpleRosterView::dnd_get_vCard(const Element& iq)
 {
-     Element* vCard = iq.findElement("vCard");
+     const Element* vCard = iq.findElement("vCard");
 
      if (vCard)
      {
