@@ -30,6 +30,7 @@
 #include <X11/Xatom.h>
 
 #include "gtkspell.h"
+#include "eggtrayicon.h"
 
 using namespace jabberoo;
 using namespace GabberUtil;
@@ -57,26 +58,26 @@ ShowDlg::ShowDlg(jabberoo::Presence::Show current_show, const string& current_st
      // Enter closes
      _thisWindow->key_press_event.connect(slot(this, &ShowDlg::on_key_press_event));
 
-     // Setup text widget
-     _txtStatus = getWidget<Gtk::Text>("Status_Status_txt");
-     _txtStatus->set_word_wrap(true);
-     _txtStatus->set_editable(true);
-     if (gtkspell_running())
-	  gtkspell_attach(_txtStatus->gtkobj()); // Attach gtkspell for spell checking
-
-     int i = 0;
-     if (current_status.length() > 0)
-	  _txtStatus->insert(current_status);
-     else
-	  _txtStatus->insert(_("Online"));
-
-     // Display the show
-     getLabel("Status_Show_txt")->set_text(getShowName(_curShow));
-
      // Show pixmap
      string show_pix = _pix_path + getShowFilename(_curShow) + ".xpm";
      getWidget<Gnome::Pixmap>("Status_Show_pix")->load(show_pix);
 
+     // Display the show
+     getLabel("Status_Show_txt")->set_text(getShowName(current_show));
+
+     // Setup status text widget
+     _txtStatus = getWidget<Gtk::Text>("Status_Status_txt");
+     _txtStatus->set_word_wrap(true);
+     _txtStatus->set_editable(true);
+     _txtStatus->changed.connect(slot(this, &ShowDlg::on_txtStatus_changed));
+     if (gtkspell_running())
+	  gtkspell_attach(_txtStatus->gtkobj()); // Attach gtkspell for spell checking
+
+     if (current_status.length() > 0)
+	  _txtStatus->insert(fromUTF8(_thisWindow, current_status)); // text encoding is local if displaying/inputting, UTF-8 otherwise.
+     else
+	  _txtStatus->insert(getShowName(current_show));
+
      // Display the priority
      _spinPriority->set_value(priority);
 
@@ -112,11 +113,6 @@ void ShowDlg::on_UpdateStatus_clicked()
 
 void ShowDlg::on_Cancel_clicked()
 {
-     // Grab the old settings
-     G_Win->display_status(G_App->getCfg().get_show(), 
-			  G_App->getCfg().get_status(),
-			  G_App->getCfg().get_priority());
-
      close();
 }
 
@@ -139,12 +135,17 @@ gint ShowDlg::on_key_press_event(GdkEventKey* e)
      return 0;
 }
 
+void ShowDlg::on_txtStatus_changed()
+{
+     getButton("Status_UpdateStatus_btn")->set_sensitive(_txtStatus->get_length() != 0);
+}
 
 // ---------------------------------------------------------
 //
 // Status Indicator
 //
 // ---------------------------------------------------------
+StatusIndicator::StatusIcon* StatusIndicator::_statusIcon = NULL;
 StatusIndicator::DockletWin* StatusIndicator::_statusDock = NULL;
 StatusIndicator::StatusToolbar* StatusIndicator::_statusToolbar = NULL;
 
@@ -178,6 +179,19 @@ void StatusIndicator::display_presence(const jabberoo::Presence& p)
 	  _statusToolbar = manage(new StatusToolbar(G_Win));
      _statusToolbar->display_presence(p);
 
+     // Status Icon
+     if (G_App->getCfg().statusicon.show)
+     {
+	  if (_statusIcon != NULL)
+	       _statusIcon->update_status(p);
+	  else
+	       _statusIcon = manage(new StatusIcon(p));
+     }
+     else if (_statusIcon != NULL)
+     {
+	  _statusIcon->on_close();
+     }       
+
      // Docklet
      if (G_App->getCfg().docklet.show)
      {
@@ -200,6 +214,151 @@ Gtk::Window* StatusIndicator::get_dock_window()
 	  return NULL;
 }
 
+// ---------------------------------------------------------
+// Status Icon (freedesktop.org protocol)
+// ---------------------------------------------------------
+StatusIndicator::StatusIcon::StatusIcon(const jabberoo::Presence& p)
+     : _flash_events(false), _need_refresh(false), 
+       _current_show(getShowFilename(p.getShow())), 
+       _pix_path(ConfigManager::get_PIXPATH())
+{
+     if (p.getType() == jabberoo::Presence::ptInvisible)
+	  _current_show = "invisible";
+
+     // Freedesktop.org protocol. Woo!!
+
+     // make status docklet a new window with a fixed size
+     _status_icon = Gtk::wrap(GTK_CONTAINER(egg_tray_icon_new(_("gabber"))));
+     _status_icon->add_events(GDK_BUTTON_PRESS_MASK);
+     _status_icon->button_press_event.connect(slot(this, &StatusIcon::on_button_press));
+     _status_icon->realize();
+     _status_icon->delete_event.connect(slot(this, &StatusIcon::on_window_delete));
+     
+     // Add the frame
+     _frmQueue = manage(new Gtk::Frame());
+     _frmQueue ->set_shadow_type(GTK_SHADOW_NONE);
+     _status_icon->add(*_frmQueue);
+     
+     // Give it the proper pixmap
+     update_status(p);
+
+     Gtk::Main::timeout.connect(slot(this, &StatusIcon::FlashEvents), 500);
+
+     _frmQueue->show();
+     _status_icon->show();
+}
+
+StatusIndicator::StatusIcon::~StatusIcon()
+{
+}
+
+gint StatusIndicator::StatusIcon::on_window_delete(GdkEventAny *e)
+{
+     on_close();
+
+     return 0;
+}
+
+void StatusIndicator::StatusIcon::on_close()
+{
+     // Kill the status docklet
+     if (_status_icon != NULL && _statusIcon != NULL)
+     {
+	  _status_icon->destroy();
+	  delete _statusIcon;
+	  _statusIcon = NULL;
+     }
+}
+
+void StatusIndicator::StatusIcon::update_status(const jabberoo::Presence& p)
+{
+     _current_show = getShowFilename(p.getShow());
+     if (p.getType() == jabberoo::Presence::ptInvisible)
+	  _current_show = "invisible";
+
+     // Grab the pixmap for current show
+     string status_file = _pix_path + _current_show + ".xpm";
+     Gdk_Pixmap icon_pixmap(_frmQueue->get_window(), 
+			    _icon_mask,
+			    Gdk_Color("white"),
+			    status_file);
+     Gtk::Widget* icon = manage(new Gtk::Pixmap(icon_pixmap, _icon_mask));
+     _frmQueue->remove();
+     _frmQueue->add(*icon);
+     icon->show();
+
+     _tips.set_tip(*_status_icon, getShowName(p.getShow()) + ": " + fromUTF8(_status_icon, p.getStatus()));
+}
+
+int StatusIndicator::StatusIcon::on_button_press(GdkEventButton* e)
+{
+     // Determine if we should show/hide the main window or popup menu
+     if (e->type == GDK_BUTTON_PRESS)
+     {
+          MessageManager::EventList& events = G_App->getMessageManager().getEvents();
+          if (!events.empty() && e->button == 1)
+          {
+	       _frmQueue->set_shadow_type(GTK_SHADOW_IN);
+               G_App->getMessageManager().display(events.begin()->first, events.begin()->second);
+               G_Win->refresh_roster();
+          }
+	  else if (e->button == 1)
+	       G_Win->toggle_visibility();
+	  else if (e->button == 3)
+	  {
+	       G_Win->get_menuDocklet()->show_all();
+	       G_Win->get_menuDocklet()->popup(e->button, e->time);
+	  }
+     }
+
+     return 0;
+}
+
+gint StatusIndicator::StatusIcon::FlashEvents()
+{
+     MessageManager& mm = G_App->getMessageManager();
+     MessageManager::EventList& events = mm.getEvents();
+     MessageManager::EventList::iterator it = events.begin();
+     ConfigManager& cfgm = G_App->getCfg();
+
+     if (it == events.end() && _need_refresh)
+     {
+          // Grab the pixmap for current show
+          string status_file = _pix_path + _current_show + ".xpm";
+          Gdk_Pixmap icon_pixmap(_frmQueue->get_window(),
+                                 _icon_mask,
+                                 Gdk_Color("white"),
+                                 status_file);
+          Gtk::Widget* icon = manage(new Gtk::Pixmap(icon_pixmap, _icon_mask));
+          _frmQueue->remove();
+          _frmQueue->add(*icon);
+	  _frmQueue->set_shadow_type(GTK_SHADOW_NONE);
+          icon->show();
+	  _need_refresh = false;
+     }
+     else if (it != events.end())
+     {
+	  _need_refresh = true;
+          if (_flash_events)
+          {
+               Gtk::Widget* icon = manage(new Gtk::Pixmap(mm.getEventPixmap(it->second), mm.getEventBitmap(it->second)));
+               _frmQueue->remove();
+               _frmQueue->add(*icon);
+	       _frmQueue->set_shadow_type(GTK_SHADOW_OUT);
+               icon->show();
+          }
+          else
+          {
+               Gtk::Widget* icon = manage(new Gtk::Pixmap(mm.getEventPixmap(0), mm.getEventBitmap(0)));
+               _frmQueue->remove();
+               _frmQueue->add(*icon);
+               icon->show();
+          }
+     }
+     _flash_events = !_flash_events;
+     return TRUE;
+}
+
 // ---------------------------------------------------------
 // Docklet Window
 // ---------------------------------------------------------
@@ -448,8 +607,8 @@ void StatusIndicator::StatusToolbar::display_presence(const jabberoo::Presence&
      }
      if (x)
      {
-	  string song_title = x->findElement_getCDATA("title");
-	  string state = x->findElement_getCDATA("state");
+	  string song_title = x->getChildCData("title");
+	  string state = x->getChildCData("state");
 	  if (state == "stopped")
 	  {
 	       song_title += _(" [stopped]");
