@@ -34,15 +34,13 @@
 #include <netdb.h>
 #include <iostream>
 #include <string>
-#ifdef USING_STD_SSTREAM
+#ifdef HAVE_STD_SSTREAM
 #include <sstream>
 #else
-#include <stdstream>
+#include <strstream>
 #endif
-#include <sys/socket.h>
 #include <arpa/inet.h>
 #include <errno.h>
-#include <netdb.h>
 #include <sys/wait.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -54,9 +52,17 @@ using namespace std;
      GStaticMutex TCPTransmitter::_gethostbyname_mutex = G_STATIC_MUTEX_INIT;
 #endif // HAVE_GETHOSTBYNAME_R_GLIB_MUTEX
 
-#ifdef MACOSX
+#ifndef HAVE_SOCKLEN_T
+#ifdef USE_SOCKLEN_T_SIZE_T
+typedef size_t socklen_t;
+#else // USE_SOCKLEN_T_INT is the only other case
 typedef int socklen_t;
-#endif // MACOSX
+#endif // USE_SOCKLEN_T_SIZE_T
+#endif // HAVE_SOCKLEN_T
+
+#ifdef SOLARIS
+#define INET_ADDRSTRLEN 16
+#endif // SOLARIS
 
 TCPTransmitter::TCPTransmitter():
      _socketfd(-1), _IOChannel(NULL), _reconnect_timeoutid(0),
@@ -818,7 +824,7 @@ gboolean TCPTransmitter::on_host_resolved(GIOChannel* iochannel, GIOCondition co
 		    transmitter._hostResolved = true;
 	       }
 	       
-	       delete buffer;
+	       delete[] buffer;
 	  }
 	  else 
 	  {
@@ -1338,7 +1344,7 @@ const string TCPTransmitter::getsockname()
      }
 }
 
-void TCPTransmitter::setProxy(const string &ptype, const string &host, guint port, const string &user, const string &password, bool tryOther = true)
+void TCPTransmitter::setProxy(const string &ptype, const string &host, guint port, const string &user, const string &password, bool tryOther)
 {
      if (_state != Offline)
      {
@@ -1570,7 +1576,7 @@ void TCPTransmitter::proxyHandleSocks5ConnectReply()
 
 void TCPTransmitter::proxySendSocks5Auth()
 {
-#ifdef USING_STD_SSTREAM
+#ifdef HAVE_STD_SSTREAM
      ostringstream header;
 #else
      ostrstream header;
@@ -1608,7 +1614,7 @@ void TCPTransmitter::proxySendSocks5Auth()
 
 void TCPTransmitter::proxySendSocks5Connect()
 {
-#ifdef USING_STD_SSTREAM
+#ifdef HAVE_STD_SSTREAM
      ostringstream header;
 #else
      ostrstream header;
@@ -1687,10 +1693,10 @@ void TCPTransmitter::proxyHandleHttpHead()
      // have we already read the response code?
      if (!_proxy.response)
      {
-#ifdef USING_STD_SSTREAM
+#ifdef HAVE_STD_SSTREAM
 	  istringstream ist(_proxy.response_line);
 #else
-	  istrstream ist(_proxy.response_line);
+	  istrstream ist(_proxy.response_line.c_str());
 #endif
 	  string skipstring;
 
@@ -1781,7 +1787,7 @@ void TCPTransmitter::proxyHandleHttpHead()
 
 void TCPTransmitter::proxyHandshakeOut()
 {
-#ifdef USING_STD_SSTREAM
+#ifdef HAVE_STD_SSTREAM
      ostringstream proxy_head;
 #else
      ostrstream proxy_head;
